/*================= Notes =================================
name: integrin + agents
author: Daniyal Motan Jun 2016 adapted from Aban Shuaib
date: 13062016

This code is for the interaction of six agents Integrin, FAK, Ras, Raf, MEK and Erk1/2. Integrin are on the cell membrane and are in either dormant or active states
(they switch from dormant - Integrin(00)to active aIntegrin(01) due to application of mechanical stimuli then revert back to dormancy after a certain number of iterations). 
The active state of Integrin interacts with free dormant FAK (10) while the inactive form doesn't do anything.
After interaction of FAK with the activated integrin - an Integrin FAK complex (abIntegrin(03))is formed and FAK becomes activated and bound - abFAK (11).
abFAK then has the ability to interact with Ras (free dormant Ras(20)) and form a bond. On doing so abFAk(11) changes state to ab2FAK(12) while Ras(20) changes state to abRas(21).
After a number of iterations abRas(21) changes state to aRas(22) and dissociates from  ab2FAK(12).
This could also be achieved if either the abIntegrin (03) reverts to dormancy - Integrin(00),
or ab2FAK(12) becomes dormant - FAK(10) - after a certain number of iterations.
aRas(22) is now free in the cytoplasm and ready to interact with dormant Raf(30).
aRas(22) may also revert back to dormant Ras(20) after a certain number of iterations.

Cell
		1000 - MSC undifferentiated
		
	Integrin 
		00 - dormant (no mechanical stimulation) - free - Integrin
		04 - active (but mechanical stimulation below threshold or no mech stimulation at all) - free - Integrin
        01 - fully active (with mechanical stimulation aboove threshold)- free - aInegrin (a5 binding domain needs mechanical tension to become accessible to FAK)
        02 - fully active (with mechanical stimulation above threshold)- bound to FAK - abIntegrin
	
	FAK
		10 - free in cytoplasm - available - FAK
		11 - bound to Integrin - active - abFAK
		12 - bound to Integrin and Ras - active - ab2FAK
		13 - free in cytoplasm dormant/not available - dFAK
		
	
	Ras
		20 - free in cytoplasm - dormant - Ras
		21 - bound to FAK integrin complex - active - abRas
		22 - dissociated from FAK integrin complex - active - aRas
		23 - bound to Raf - active - ab2Ras
		
	Raf
		30 - free in cytoplasm - dormant - Raf
		31 - bound to Ras - active - abRaf
		32 - dissociated from Ras - active - aRaf
		33 - bound to MEK - active - ab2Raf
	
	MEK
		40 - free in cytoplasm - dormant - MEK
		41 - bound to Raf - active - abMEK
		42 - dissociated from Raf - active - aMEK
		43 - bound to Erk1/2 - active - ab2MEK
	
	Erk
		50 - free in cytoplasm - dormant - Erk1/2
		51 - bound to MEK - active - abErk1/2
		52 - dissociated from MEK - active - aErk1/2 (ready to interact with nuclear receptors and go on to activate Runx2)
		53 - bound to Runx2 - ab2Erk
		54 - bound to other molecular complexes (atleast for emergent phenomena we are targetting) - ab3Erk
		
	Runx2 (trancription factor)
		60 - free in nucleoplasm - unphosphorylated - Runx2
		61 - bound to Erk1/2 - abRunx2
		62 - free in nucleoplasm - phosphorylated - aRunx2
		63 - bound to transcription factor/DNA complex - ab2Runx2
		
	Complex (of Osx and other transcription factors with DNA) - <<Black Box>>
		70 - monomeric Osx
		700 - monomeric Osx bound to Runx2 
		71 - monomeric Osx + DNA
		710 - monomeric Osx + DNA bound to Runx2
		711 - monomeric Osx + DNA bound to Runx2 + AP1 (c-fos/c-jun complex) (active transcribing)
		72 - multimeric complex of Osx + other transcription factors
		720 - multimeric complex of Osx + other transcription factors bound to Runx2
		73 - multimeric complex + DNA
		730 - multimeric complex + DNA bound to Runx2(active transcribing)
		
	AP1 (Complex of stable cfos and cjun + other transcription factors) - Produced after aErk conc in nucleus crosses threshold
		  (since its a complex of Immediate Early Gene Products - takes relatively less time to produce)
		80 - free in nucleoplasm - active aAP-1
		81 - bound to monomeric complex of OSx + DNA + Runx2 
	
	OPN (Osteopontin mRNA)
		90 - free in nucleo or cyto
		91 - bound to Ribosome - available to other Ribosomes
		92 - bound to Ribosome - unavailable to other Ribosomes
		
	OCN (Osteocalcin mRNA)
		100 - free in nucleo or cyto
		101 - bound to Ribosome - available to other Ribosomes
		102 - bound to Ribosome - unavailable to other Ribosomes
		
	ALP(Alkaline phosphatase mRNA)
		110 - free in nucleo or cyto
		111 - bound to Ribosome - available to other Ribosomes
		112 - bound to Ribosome - unavailable to other Ribosomes
		
	BSP (Bone sialoprotein mRNA)
		120 - free in nucleo or cyto
		121 - bound to Ribosome - available to other Ribosomes
		122 - bound to Ribosome - unavailable to other Ribosomes
		
	Ribosome
	130 - incomplete ribosome
	131 - incomplete ribosome bound to mRNA
	140 - complete ribosome - can translate
	141 - complete ribosome bound to mRNA - translating
	
	Matrix proteins
	150 - Bieng transported in vesicle
	151 - deposited outside cell - fixed in place
	
		
	Dissociation/Delay/Reactivation Constants --> Activation Cycle Switch (ACS) 
	
		recdelay - Integrin(00) to aIntegrin(01)
		ACS1 - abIntegrin (02) to Integrin (00)
		
		ACSFAK - dFAK (13) to FAK (10)
		ACS2 - abFAK (11) to dFAK (13)
		ACS3 - ab2FAK (12) to abFAK (11)
		
		ACS4 - abRas (21) to aRas (22)
		ACS5 - ab2Ras (23) to aRas (22)
		ACS6 - aRas (22) to Ras (20)
		
		ACS7 - abRaf (31) to aRaf (32)
		ACS8 - ab2Raf (33) to aRaf (32)
		ACS9 - aRaf (32) to Raf (30)
		
		ACS10 - abMEK (41) to aMEK (42)
		ACS11 - ab2MEK (43) to aMEK (42)
		ACS12 - aMEK (42) to MEK (40)
		
		ACS13- abErk (51) to aErk (52)
		ACS14- aErk (52) to Erk (50)
		ACS15- ab2Erk (53) to aErk (52)
		ACS16- ab3Erk (54) to aErk (52)
	
		ACS17 - abRunx2 (61) to aRunx2 (62)
		ACS18 - ab2Runx2 (63) to aRunx2 (62)
		ACS19 - aRunx2 (62) to Runx2 (60)
		
		ACS20 - aMonoComplex (700) to MonoComplex (70)
		ACS21 - aMonoComplexDNA (710) to MonoComplexDNA (71)
		ACS22 - aMonoComplexDNA+AP-1 (711) to MonoComplexDNA (71)
		ACS23 - aMultiComplex (720) to MultiComplex (72)
		ACS24 - aMultiComplexDNA (730) to MultiComplexDNA (73)
		
		ACS25 - unavailable bound OPN mRNA (92) to available bound OPN mRNA (91)
		ACS26 - available bound OPN mRNA (91) to unbound OPN mRNA (90)
		
		ACS27 - unavailable bound OCN mRNA (102) to available bound OCN mRNA (101)
		ACS28 - available bound OCN mRNA (101) to unbound OCN mRNA (100)
		
		ACS29 - unavailable bound ALP mRNA (112) to available bound ALP mRNA 111)
		ACS30 - available bound ALP mRNA (111) to unbound ALP mRNA (110)
		
		ACS31 - unavailable bound BSP mRNA (122) to available bound BSP mRNA (121)
		ACS32 - available bound BSP mRNA (121) to unbound BSP mRNA (120)
		
		ACS33 - bound incomplete ribosome (131) to unbound incomplete ribosome (130)
		ACS34 - bound complete ribosome (141) to unbound complete ribososme (140)
		
	Special Constants
		pErkcount - counts the number of phosphorylated Erk in cyto, similarly other counters can be added to play the role of global agents
		OCNproductid - to assign ids to OCN mRNA produced de novo by RSK pathway ,starts with 100 so as to differentiate from OCN produced by other pathways
		OCNACS - delay counter for restarting of OCN transcription by RSK pathway
		OCNgeneratedelay - counter for generation of OCN via RSK phosphorylation pathway
		
		OCNproteindelay
		OPNproteindelay
		ALPproteindelay
		BSPproteindelay
		
		OCNproteinid
		OPNproteinid
		ALPproteinid
		BSPproteinid
		
		OCNDelay
		OPNDelay
		ALPDelay
		BSPDelay
		Runx2Delay
		ComplexDelay
		AP1Delay
		
//Important Info 		
The average human gene is about 28,000 bases long, with only about 5% of it encoding for protein, which on an average is about 450 amino acids long.
The major steps that take some time are:
Transcription [DNA → pre-mRNA] : ~20min
Splicing [pre-mRNA → mature mRNA] : 5-10min
Transport [mRNA from nucleus to cytoplasm] : 10-30min
Translation [mRNA → Protein] : 90sec
		
=== Keys ===

*/

#include "header.h"
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#define PI 3.141592
#define pi 3.141592
#define recdelay_constant 3
#define disK_constant 5

int GLOBAL_cyto_pErk_count = 0;
//int nuc_pErk_count = 0
//int pRunx2_count = 0
//int OPN_mRNA_count = 0
//int OCN_mRNA_count = 0
//int ALP_mRNA_count = 0
//int BSP_mRNA_count = 0

/* code writing started on 16/06/2016 from agent integrin */
/* For now assume Integrin  dormant at start - and stimulated after 3 iterations with a stimulation applied for 50 iterations - Therefore ACS1 at start is 3,
while ACS1 after the first 3 iterations is set at 50. Consequently 50 iteration long stimulations are applied with 3 iteration long gaps (when integrin is unstimulated/is dormant)
 */
 
int Integrin_outputdata()
{
	//extract force data from external file and update each integrin with respective force 
	{
	//declare pointer to a file	
	FILE *fp;
	int count = 0;
	//declare an array of 500 doubles to store force values
	double forc[500];
	double ind;
	//open the input file stream
	fp = fopen("force.txt", "r");
	while(count<500)
	{
		//scan through text file and extract doubles line by line
		fscanf(fp, "%lf ", &ind);
		forc[count] = ind;
		count++;
	}
	
	int i;
	//set up for loop from i values 0 - 499
	for(i = 0; i<500; i++)
		{
			//assigns correct force value to each integrin by going through th ids in order  
			if(get_id() == (i+1))
		      set_force(forc[i]);
		}
	//close the input file stream
	fclose(fp);
	}
	
	// Send location message /
	add_Integrinlocation_message(get_id(),		// agent id /
						get_posx(),		// agent x-axis position /
						get_posy(),		// agent y-axis position /
						get_posz(),		// agent z-axis position /
						get_state(),	// agent type and state /
						get_iradius());	// agent message range /
						
	/* if Integrin is dormant - unstimulated*/
	if(get_state() == 00)
	{
		if(get_ACS1() > 0)
		{
			/* Then decrement ACS1 counter */
			set_ACS1(get_ACS1() - 1);
			/* If processing has finished (ACS1 is zero) */
			if(get_ACS1() == 0)
			{
			/* Switch to active aIntegrin(04) */
				set_state(04);					
			}
		}
		/* If ACS1 is zero, re-set ACS1 to 50*/
		srand ( time(NULL) );
		int number = ((srand() % 30) + 30); 
		if(get_ACS1() == 0) set_ACS1(number);
	}
	
	/* if Integrin is active - unstimulated*/
	if(get_state() == 04)
	{
		if(get_forcetag() == 02)
		{ 									 
		//if force on particular integrin is above threshold of activation
		if(get_force() > 0.00001)
			{
			/* Switch to fully active stimulated Integrin(01) */
				set_state(01);
			}
		}
		
		if(get_forcetag() == 01)
		{
		//if force on particular integrin is above threshold of activation
		if(get_force() > 0.0005)
			{
			/* Switch to fully active stimulated Integrin(01) */
				set_state(01);	
			}
		}
		
		
	}
	
	/* If Integrin is active free or active bound to FAK */		
	if (get_state () == 01 || get_state() == 02 || get_state() == 04)
	{
		if( get_ACS1() > 0)
		{
			// Then decrement ACS1 counter /
			set_ACS1(get_ACS1() - 1);
			// If processing has finished (ACS1 is zero) /
			if(get_ACS1() == 0)
			{
				/* Change state from just active or active stimulated or active bound to dormant - unstimulated */
				set_state(00);
				srand ( time(NULL) );
				int numb = ((srand() % 40) + 40);
				set_ACS1(numb); 
			}
				
		}
	}	
		// Send unbind message /
						add_FAKunbind_message(get_id(),		// agent id /
						get_posx(),		// agent x-axis position /
						get_state());	// agent state 
	return 0;
}

int Integrin_inputdata()
{
	xmachine_memory_Integrin * xmemory = current_xmachine->xmachine_Integrin;
	
	/* Create variables to use */
	double x1, y1, x2, y2, z1, z2, closestdist;
	int id2, s1, s2, closestid, closeststate;
	double distance;
	
	/* Copy agent data to #1 variables */
	x1 = get_posx();
	y1 = get_posy();
	z1 = get_posz();
	s1 = get_state();
	/* Interaction radius of the agent (squared so don't need to sqrt compared var) */
	closestdist = get_iradius()*get_iradius();
	/* Index of closest agent to interact with and set to default as own id */
	closestid = get_id();

	/* Check for possible bindings */
	/* Get first location message */
	FAKlocation_message = get_first_FAKlocation_message();
	/* And loop through them until there are none left */
	while(FAKlocation_message)
	{
		/* If the location message is not from the current agent */
		if((FAKlocation_message->id != get_id()))
		{
			/* Copy location message data to #2 variables */
			id2 = FAKlocation_message->id;
			x2 = FAKlocation_message->x;
			y2 = FAKlocation_message->y;
			z2 = FAKlocation_message->z;
			s2 = FAKlocation_message->state;
			
			/* Calculate the distance between agent and message sending agent */
			/* Do not square root as more efficient to square interaction radius */
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			/* If distance is equal or smaller than the agent interaction radius */
			if(distance < closestdist)
			{
				/* If Integrin is active and free (01) and FAK is dormant free in cytoplasm (10) */
				if((s1 == 01 && s2 == 10))
					
				{
					/* if this agent is closer than the index of the closest */
					if(distance < closestdist)
					{
						/* update closest distance to bondable agent */
						closestdist = distance;
						/* also update the closest agent's id and state */
						closestid = id2;
						closeststate = s2;
					}
				}
			}
			
		}
		/* Get next location message */
		FAKlocation_message = get_next_FAKlocation_message(FAKlocation_message);
	}
		
	/* If closest id is not the sham value therefore a possible bond has been found */
	if(closestid != xmemory->id)
	{
		/* Send try to bond message */
		add_Integrinnewbond_message(xmemory->id,         
						xmemory->state,
						closestid,           /* id of agent message is refering to */
						3,                   /* 3 is for a try to bond */
						closestdist,        /* distance to between agents */
						get_iradius(),
						get_posx(),
						get_posy(),
						get_posz());
	}
	
	return 0;
}

int Integrin_move()
{
	xmachine_memory_Integrin * xmemory = current_xmachine->xmachine_Integrin;
	
	/* Check if I have been bound */
	/* Get first bond message */
	FAKfinalbond_message = get_first_FAKfinalbond_message();
	/* And loop through messages until there are none left */
	while(FAKfinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(FAKfinalbond_message->idto == xmemory->id )
		{

		/* If agent Integrin is active, stimulated, and free. Change state to stimulated and bound - abIntegrin(02)*/
		if(get_state()== 01  && FAKfinalbond_message->statefrom == 11) 
						{
						set_state(02); 
						}
		}
		/* Get next bond message */
		FAKfinalbond_message = get_next_FAKfinalbond_message(FAKfinalbond_message);
	}

	return 0;
}


int FAK_outputdata()
{
	/* Send location message */
	add_FAKlocation_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state(),	/* agent type and state */
						get_iradius());	/* agent message range */
						
	/* if FAK is bound to integrin and active - abFAK(11) */
	if(get_state() == 11)
	{
		if(get_ACS2() > 0)
		{
			/* Then decrement ACS2 counter */
			set_ACS2(get_ACS2() - 1);
			/* If processing has finished (ACS1 is zero) */
			if(get_ACS2() == 0)
			{
			/* Switch to dormant unbound dFAK */
				set_state(13);
				set_ACSFAK(5);
			}
		}
		if(get_ACS2() == 0 || get_ACS2() < 0)
			{
			/* Switch to dormant unbound dFAK */
				set_state(13);
				set_ACSFAK(5);
			}
		/* ACS2 will only reset when a bond between integrin and FAK has been formed*/
	}
	
	/* If FAK is bound to both integrin and RAS and is active - ab2FAK (12)*/		
	if (get_state () == 12)
	{
		if( get_ACS3() > 0)
		{
			// Then decrement ACS3 counter /
			set_ACS3(get_ACS3() - 1);
			// If processing has finished (ACS3 is zero) /
			if(get_ACS3() == 0)
			{
				/* Change state from being active + bound to both RAS and Integrin - ab2FAK(12) to active + only bound to Integrin - abFAK */
				set_state(11);
				set_ACS2(get_ACS2() - 7);// to account for overall reduction in ACS2 even while FAK in state 12 - bound to Ras/
				/*ACS(3) will only reset when a bond between FAK-Integrin complex and RAS has formed */ 

			}
				
		}
	}	
	
	/* if FAK is dormant and free in cytoplasm - dFAK(13) */
	if(get_state() == 13)
	{
		if(get_ACSFAK() > 0)
		{
			/* Then decrement ACSFAK counter */
			set_ACSFAK(get_ACSFAK() - 1);
			/* If processing has finished (ACSFAK is zero) */
			if(get_ACSFAK() == 0)
			{
			/* Switch to unbound FAK available to bind to integrin*/
				set_state(10);
			}
		}
		/* ACSFAK will only reset when abFAK unbinds from Integrin */
	}
	return 0;
}

int FAK_inputdata()
{
	xmachine_memory_FAK * xmemory = current_xmachine->xmachine_FAK;
	
	
	//Get first FAKunbind message /
	FAKunbind_message = get_first_FAKunbind_message();
		while(FAKunbind_message)
	{
		// If the Integrin sending the message has gone to state 00 /
		if((FAKunbind_message->state == 00) && (FAKunbind_message->posx == get_posx()))
		{
			//set FAK to unbound dormant state (13) /
			set_state(13);
			set_ACSFAK(5);
		}	
		// Get next location message /
		FAKunbind_message = get_next_FAKunbind_message(FAKunbind_message);
		
	}
		
		
	/* Create variables to use */
	double x1, y1, x2, y2, z1, z2, closestdist;
	int id2, s1, s2, closestid, closeststate; 
	double distance;
		
	/* Copy agent data to #1 variables */
	x1 = get_posx();
	y1 = get_posy();
	z1 = get_posz();
	s1 = get_state();
	
	/* Interaction radius of the agent (squared so don't need to sqrt compared variable) */
	closestdist = get_iradius()*get_iradius();
	/* Index of closest agent to interact with and set to default as own id */
	closestid = get_id();


	 
	// Check for possible bindings /
	//Get first location message /
	Integrinlocation_message = get_first_Integrinlocation_message();
	// And loop through them until there are none left /
	

	while(Integrinlocation_message)
	{
		// If the location message is not from the current agent /
		if((Integrinlocation_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = Integrinlocation_message->id;
			x2 = Integrinlocation_message->x;
			y2 = Integrinlocation_message->y;
			z2 = Integrinlocation_message->z;
			s2 = Integrinlocation_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				if(
				// if agent FAK is dormant and free, and Integrin is active and free  /
				(s1 == 10 && s2 == 01) 				 

				)

				{
		 
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		// Get next location message /
		Integrinlocation_message = get_next_Integrinlocation_message(Integrinlocation_message);
		
	}
	
	
	/* Check for possible bindings */
	/* Get first location message */
	Raslocation_message = get_first_Raslocation_message();
	/* And loop through them until there are none left */
	
	while(Raslocation_message) 
	{
		/* If the location message is not from the current agent */
		if((Raslocation_message->id != get_id()))
		{
			/* Copy location message data to #2 variables */
			id2 = Raslocation_message->id;
			x2 = Raslocation_message->x;
			y2 = Raslocation_message->y;
			z2 = Raslocation_message->z;
			s2 = Raslocation_message->state;
			
			/* Calculate the distance between agent and message sending agent */
			/* Do not square root as more efficient to square interaction radius */
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			/* If distance is equal or smaller than the agent interaction radius */
			if(distance < closestdist)
			{
				if(
				/* if agent FAK is bound to Integrin and active - abFAK, and, Ras is dormant free in cytoplasm - Ras */
				(s1 == 11 && s2 == 20) 				
				)

				{
					/* if this agent is closer than the index of the closest */
					if(distance < closestdist)
					{
						/* update closest distance to bondable agent */
						closestdist = distance;
						/* also update the closest agent's id and state */
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		/* Get next location message */
		Raslocation_message = get_next_Raslocation_message(Raslocation_message);
	}	
	
	// If closest id is not the sham value therefore a possible bond has been found /
	if(closestid != xmemory->id)
	{
		// Send try to bond message /
		add_FAKnewbond_message(xmemory->id,         
						xmemory->state,
						closestid,           // id of agent message is refering to /
						3,                   // 3 is for a try to bond /
						closestdist,        // distance to between agents /
						get_iradius(),
						get_posx(),
						get_posy(),
						get_posz());
	}
	
	return 0;
}

int FAK_checkbondtries()
{
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_FAK * xmemory = current_xmachine->xmachine_FAK;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	double xIntegrin, yIntegrin, zIntegrin;
	
	/* Check if I have been bound to Integrin */
	/* Get first bond message from Integrin agent*/
	Integrinnewbond_message = get_first_Integrinnewbond_message();
	/* And loop through messages until there are none left */
	while(Integrinnewbond_message)
	{
		if(Integrinnewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (Integrinnewbond_message->distance < closestdist))
			{
				closestdist = Integrinnewbond_message->distance;
				closestid = Integrinnewbond_message->idfrom;
				closeststate = Integrinnewbond_message->statefrom;
				xIntegrin = Integrinnewbond_message->x; 
				yIntegrin = Integrinnewbond_message->y;
				zIntegrin = Integrinnewbond_message->z;
			}
		}
		
		/* Get next trybond message */
		Integrinnewbond_message = get_next_Integrinnewbond_message(Integrinnewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						/* If FAK free in cyto and Integrin is free and active, set FAK to abFAK and Integrin to abIntegrin 
						FAK has moved to Integrin and come in contact --- this movement is assumed to be instantaneous 
						Also set ACS2 to 30 --  relative to other dissociation half life values-*/
						if(get_state() == 10  && closeststate == 01) 
						{
							set_state(11);											
							set_ACS2(30);
							
	                    /*  Assuming instantaneous movement--repositioning of FAK so that it is in contact with Integrin that its bound to */		
							set_posx(xIntegrin); 
							set_posy(yIntegrin); 
							set_posz(zIntegrin);
						}
						
						/* Send bond message to molecule to bind */
		add_FAKfinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          /* id of Integrin agent message is refering to */
						3,
						closestdist,    /* distance previously not used/ now used for try */
						get_iradius(),
						xIntegrin,
						yIntegrin,
						zIntegrin);

	}
	
	//rule for dissociation of FAK from an inactive Integrin/

	return 0;
}

int FAK_move()
{
	xmachine_memory_FAK * xmemory = current_xmachine->xmachine_FAK;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving dormant FAK in cytoplam*/
	if(( xmemory->state == 10 || xmemory->state == 13  ))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
		/* If FAK active and bound to Integrin or Integrin and Ras*/
		if((xmemory->state == 11 || xmemory->state == 12))
		{
			//stay in position bound to integrin/
		}
				
	/* Check if I have been bound to Ras */
	/* Get first bond message */
	Rasfinalbond_message = get_first_Rasfinalbond_message();
	/* And loop through messages until there are none left */
	while(Rasfinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(Rasfinalbond_message->idto == xmemory->id )
		{

		/* If agent FAK is abFAK(11), Change state to ab2FAK(12)*/
		if(get_state()== 11  && Rasfinalbond_message->statefrom == 21) 
						{
						set_state(12);
						set_ACS3(7); //from literature
						}
		}
		/* Get next bond message */
		Rasfinalbond_message = get_next_Rasfinalbond_message(Rasfinalbond_message);
	}			
	
	return 0;
}

int Ras_outputdata()
{
		/* Send location message */
	add_Raslocation_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state(),	/* agent type and state */
						get_iradius());	/* agent message range */
						
	/* if Ras is dormant - free in cytoplasm*/
	
	/* if Ras is bound to FAK and active - abRas(21) */
	if(get_state() == 21)
	{
		if(get_ACS4() > 0)
		{
			/* Then decrement ACS4 counter */
			set_ACS4(get_ACS4() - 1);
			/* If processing has finished (ACS4 is zero) */
			if(get_ACS4() == 0)
			{
			/* Switch to active unbound Ras free in cyto  and set ACS6 to 60 - arbitrary  */
				set_state(22);
				set_ACS6(60);		
			}
		}
		/* ACS4 will only reset when a bond between FAK and Ras has been formed*/
	}	
	/* if Ras is bound to Raf and active - ab2Ras(23) */
	if(get_state() == 23)
	{
		if(get_ACS5() > 0)
		{
			/* Then decrement ACS5 counter */
			set_ACS5(get_ACS5() - 1);
			/* If processing has finished (ACS5 is zero) */
			if(get_ACS5() == 0)
			{
			/* Switch to active unbound Ras free in cyto  and set ACS5 to previous ACS6 - arbitrary value  */
				set_state(22);	
			}
		}
		/* ACS5 will only reset when a bond between Ras and Raf has been formed*/
	}	
	/* if Ras is active and free in cyto - aRas(22) */
	if(get_state() == 22)
	{
		if(get_ACS6() > 0)
		{
			/* Then decrement ACS6 counter */
			set_ACS6(get_ACS6() - 1);
			/* If processing has finished (ACS6 is zero) */
			if(get_ACS6() == 0)
			{
			/* Switch to dormant Ras free in cyto Ras(20)*/
				set_state(20);		
			}
		}
		/* ACS6 will only reset when active Ras dissociates from FAK*/
	}	
	return 0;
}	

int Ras_inputdata()
{
	xmachine_memory_Ras * xmemory = current_xmachine->xmachine_Ras;
	/* Create variables to use */
	double x1, y1, x2, y2, z1, z2, closestdist;
	int id2, s1, s2, closestid, closeststate; 
	double distance;
		
	/* Copy agent data to #1 variables */
	x1 = get_posx();
	y1 = get_posy();
	z1 = get_posz();
	s1 = get_state();
	
	/* Interaction radius of the agent (squared so don't need to sqrt compared variable) */
	closestdist = get_iradius()*get_iradius();
	/* Index of closest agent to interact with and set to default as own id */
	closestid = get_id();	
	
	// Check for possible bindings /
	// Get first location message /
	Raflocation_message = get_first_Raflocation_message();
	// And loop through them until there are none left /
	
	while(Raflocation_message) 
	{
		// If the location message is not from the current agent /
		if((Raflocation_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = Raflocation_message->id;
			x2 = Raflocation_message->x;
			y2 = Raflocation_message->y;
			z2 = Raflocation_message->z;
			s2 = Raflocation_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				if(
				// if agent Ras is free in cyto and active - aRas, and, Raf is dormant free in cytoplasm - Raf /
				(s1 == 22 && s2 == 30) 				
				)

				{
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		// Get next location message /
		Raflocation_message = get_next_Raflocation_message(Raflocation_message);
	}	
	
	// If closest id is not the sham value therefore a possible bond has been found /
	if(closestid != xmemory->id)
	{
		// Send try to bond message /
		add_Rasnewbond_message(xmemory->id,         
						xmemory->state,
						closestid,           // id of agent message is refering to /
						3,                   // 3 is for a try to bond /
						closestdist,        // distance to between agents /
						get_iradius(),
						get_posx(),
						get_posy(),
						get_posz());
	}
	
	return 0;
}

int Ras_checkbondtries()
{
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_Ras * xmemory = current_xmachine->xmachine_Ras;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	double xFAK, yFAK, zFAK;
	
	/* Check if I have been bound to FAK */
	/* Get first bond message from FAK agent*/
	FAKnewbond_message = get_first_FAKnewbond_message();
	/* And loop through messages until there are none left */
	while(FAKnewbond_message)
	{
		if(FAKnewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (FAKnewbond_message->distance < closestdist))
			{
				closestdist = FAKnewbond_message->distance;
				closestid = FAKnewbond_message->idfrom;
				closeststate = FAKnewbond_message->statefrom;
				xFAK = FAKnewbond_message->x;
				yFAK = FAKnewbond_message->y;
				zFAK = FAKnewbond_message->z;
			}
		}
		
		/* Get next trybond message */
		FAKnewbond_message = get_next_FAKnewbond_message(FAKnewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						/* If Ras free in cyto and dormant and Integrin is bound to FAK and active, set Ras to abRas and FAK to ab2FAK 
						Also set ACS4 to 7 -- aritrary  --*/
						if(get_state() == 20  && closeststate == 11) 
						{
							set_state(21);											
							set_ACS4(7);
							
	                    /*  Assuming instantaneous movement--repositioning of Ras so that it is in contact with FAK that its bound to */		
							set_posx(xFAK); 
							set_posy(yFAK); 
							set_posz(zFAK);
						}
						/* Send bond message to molecule to bind */
		add_Rasfinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          /* id of FAK agent message is refering to */
						3,
						closestdist,    /* distance previously not used/ now used for try */
						get_iradius(),
						xFAK,
						yFAK,
						zFAK);

	}

	return 0;
}

int Ras_move()
{
	xmachine_memory_Ras * xmemory = current_xmachine->xmachine_Ras;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double compradius = 950.0;     /* approximate compartment radius of where Ras is to be restricted in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving dormant or active Ras in cytoplam*/
	if(( xmemory->state == 20 || xmemory->state == 22 ))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the compartment boundary */
			if(xmemory->posr < compradius)
			{
				/* Mirror position back into correct compartment */
				xmemory->posr = (2*compradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
		/* If Ras active and bound to FAK-Integrin complex or Raf*/
		if((xmemory->state == 21 || xmemory->state == 23))
		{
			//stay in position bound to integrin/
		}
				
	/* Check if I have been bound to Raf */
	/* Get first bond message */
	Raffinalbond_message = get_first_Raffinalbond_message();
	/* And loop through messages until there are none left */
	while(Raffinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(Raffinalbond_message->idto == xmemory->id )
		{

		/* If agent Ras is aRas(22), Change state to ab2Ras(23)*/
		if(get_state()== 22 ) 
						{
						set_state(23);
						set_ACS5(14); //arbitrary
						}
		}
		/* Get next bond message */
		Raffinalbond_message = get_next_Raffinalbond_message(Raffinalbond_message);
	}				
	
	return 0;
}	

int Raf_outputdata()
{
		/* Send location message */
	add_Raflocation_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state(),	/* agent type and state */
						get_iradius());	/* agent message range */
						
	/* if Raf is dormant - free in cytoplasm*/
	
	/* if Raf is bound to Ras and active - abRaf(31) */
	if(get_state() == 31)
	{
		if(get_ACS7() > 0)
		{
			/* Then decrement ACS7 counter */
			set_ACS7(get_ACS7() - 1);
			/* If processing has finished (ACS7 is zero) */
			if(get_ACS7() == 0)
			{
			/* Switch to active unbound Ras free in cyto  and set ACS9 to 60 */
				set_state(32);
				set_ACS9(60);		
			}
		}
		/* ACS9 will only reset when a bond between Ras and Raf has been formed*/
	}	
	/* if Ras is dormant - free in cytoplasm*/
	
	
	/* if Raf is bound to MEK and active - ab2Raf(33) */
	if(get_state() == 33)
	{
		if(get_ACS8() > 0)
		{
			/* Then decrement ACS8 counter */
			set_ACS8(get_ACS8() - 1);
			/* If processing has finished (ACS8 is zero) */
			if(get_ACS8() == 0)
			{
			/* Switch to active unbound Ras free in cyto  and set ACS5 to previous ACS6 */
				set_state(32);	
			}
		}
		/* ACS8 will only reset when a bond between Raf and MEK has been formed*/
	}	
	/* if Raf is active and free in cyto - aRaf(32) */
	if(get_state() == 32)
	{
		if(get_ACS9() > 0)
		{
			/* Then decrement ACS9 counter */
			set_ACS9(get_ACS9() - 1);
			/* If processing has finished (ACS9 is zero) */
			if(get_ACS9() == 0)
			{
			/* Switch to dormant Raf free in cyto Raf(30)*/
				set_state(30);		
			}
		}
		/* ACS9 will only reset when active Ras dissociates from FAK*/
	}	
	return 0;
}	

int Raf_inputdata()
{
	xmachine_memory_Raf * xmemory = current_xmachine->xmachine_Raf;
	/* Create variables to use */
	double x1, y1, x2, y2, z1, z2, closestdist;
	int id2, s1, s2, closestid, closeststate; 
	double distance;
		
	/* Copy agent data to #1 variables */
	x1 = get_posx();
	y1 = get_posy();
	z1 = get_posz();
	s1 = get_state();
	
	/* Interaction radius of the agent (squared so don't need to sqrt compared variable) */
	closestdist = get_iradius()*get_iradius();
	/* Index of closest agent to interact with and set to default as own id */
	closestid = get_id();

	// Check for possible bindings /
	// Get first location message /
	MEKlocation_message = get_first_MEKlocation_message();
	// And loop through them until there are none left /
	
	while(MEKlocation_message) 
	{
		// If the location message is not from the current agent /
		if((MEKlocation_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = MEKlocation_message->id;
			x2 = MEKlocation_message->x;
			y2 = MEKlocation_message->y;
			z2 = MEKlocation_message->z;
			s2 = MEKlocation_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				if(
				// if agent Raf is free in cyto and active - aRaf, and, MEK is dormant free in cytoplasm - MEK /
				(s1 == 32 && s2 == 40) 				
				)

				{
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		// Get next location message /
		MEKlocation_message = get_next_MEKlocation_message(MEKlocation_message);
	}	
	
	// If closest id is not the sham value therefore a possible bond has been found /
	if(closestid != xmemory->id)
	{
		// Send try to bond message /
		add_Rafnewbond_message(xmemory->id,         
						xmemory->state,
						closestid,           // id of agent message is refering to /
						3,                   // 3 is for a try to bond /
						closestdist,        // distance to between agents /
						get_iradius(),
						get_posx(),
						get_posy(),
						get_posz());
	}
	
	return 0;
}

int Raf_checkbondtries()
{
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_Raf * xmemory = current_xmachine->xmachine_Raf;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	double xRas, yRas, zRas;
	
	/* Check if I have been bound to Ras */
	/* Get first bond message from Ras agent*/
	Rasnewbond_message = get_first_Rasnewbond_message();
	/* And loop through messages until there are none left */
	while(Rasnewbond_message)
	{
		if(Rasnewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (Rasnewbond_message->distance < closestdist))
			{
				closestdist = Rasnewbond_message->distance;
				closestid = Rasnewbond_message->idfrom;
				closeststate = Rasnewbond_message->statefrom;
				xRas = Rasnewbond_message->x;
				yRas = Rasnewbond_message->y;
				zRas = Rasnewbond_message->z;
			}
		}
		
		/* Get next trybond message */
		Rasnewbond_message = get_next_Rasnewbond_message(Rasnewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						/* If Raf free in cyto and dormant and Ras free in cyto and active, set Raf to abRaf and Ras to ab2Ras  
						Ras has moved to Integrin and come in contact --- movement is instantaneous
						Also set ACS7 to 10 -- aritrary  --*/
						if(get_state() == 30  && closeststate == 22) 
						{
							set_state(31);											
							set_ACS7(14);
							
	                    /*  Assuming instantaneous movement--repositioning of Raf so that it is in contact with Ras that its bound to */		
							set_posx(xRas); 
							set_posy(yRas); 
							set_posz(zRas);
						}
						/* Send bond message to molecule to bind */
		add_Raffinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          /* id of Ras agent message is refering to */
						3,
						closestdist,    /* distance previously not used/ now used for try */
						get_iradius(),
						xRas,
						yRas,
						zRas);

	}

	return 0;
}

int Raf_move()
{
	xmachine_memory_Raf * xmemory = current_xmachine->xmachine_Raf;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving dormant or active Raf in cytoplam*/
	if(( xmemory->state == 30 || xmemory->state == 32 ))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
		/* If Raf active and bound to Ras-Integrin complex or Raf*/
		if((xmemory->state == 31 || xmemory->state == 33))
		{
			//stay in position bound to integrin/
		}
				
	/* Check if I have been bound to MEK */
	/* Get first bond message */
	MEKfinalbond_message = get_first_MEKfinalbond_message();
	/* And loop through messages until there are none left */
	while(MEKfinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(MEKfinalbond_message->idto == xmemory->id )
		{

		/* If agent Raf is aRaf(32), Change state to ab2Raf(33)*/
		if(get_state()== 32 ) 
						{
						set_state(33);
						set_ACS8(10); //arbitrary
						}
		}
		/* Get next bond message */
		MEKfinalbond_message = get_next_MEKfinalbond_message(MEKfinalbond_message);			
	}
	return 0;
}	

int MEK_outputdata()
{
		/* Send location message */
	add_MEKlocation_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state(),	/* agent type and state */
						get_iradius());	/* agent message range */
						
	/* if MEK is dormant - free in cytoplasm*/
	
	/* if MEK is bound to Raf and active - abMEK(31) */
	if(get_state() == 41)
	{
		if(get_ACS10() > 0)
		{
			/* Then decrement ACS10 counter */
			set_ACS10(get_ACS10() - 1);
			/* If processing has finished (ACS7 is zero) */
			if(get_ACS10() == 0)
			{
			/* Switch to active unbound Raf free in cyto  and set ACS12 to 88 - from half life value of 88s from (Fujioka 2006)*/
				set_state(42);
				set_ACS12(88);		
			}
		}
		/* ACS10 will only reset when a bond between Raf and MEK has been formed*/
	}	
	/* if MEK is dormant - free in cytoplasm*/
	
	
	/* if MEK is bound to Erk1/2 and active - ab2MEK(43) */
	if(get_state() == 43)
	{
		if(get_ACS11() > 0)
		{
			/* Then decrement ACS11 counter */
			set_ACS11(get_ACS11() - 1);
			/* If processing has finished (ACS11 is zero) */
			if(get_ACS11() == 0)
			{
			/* Switch to active unbound Raf free in cyto  and set ACS5 to previous ACS6  */
				set_state(42);	
			}
		}
		/* ACS8 will only reset when a bond between MEK and Erk1/2 has been formed*/
	}	
	/* if MEK is active and free in cyto - aMEK(42) */
	if(get_state() == 42)
	{
		if(get_ACS12() > 0)
		{
			/* Then decrement ACS12 counter */
			set_ACS12(get_ACS12() - 1);
			/* If processing has finished (ACS12 is zero) */
			if(get_ACS12() == 0)
			{
			/* Switch to dormant MEK free in cyto MEK(40)*/
				set_state(40);		
			}
		}
		/* ACS12 will only reset when active MEK dissociates from Raf*/
	}	
	return 0;
}	

int MEK_inputdata()
{
	xmachine_memory_MEK * xmemory = current_xmachine->xmachine_MEK;
	/* Create variables to use */
	double x1, y1, x2, y2, z1, z2, closestdist;
	int id2, s1, s2, closestid, closeststate; 
	double distance;
		
	/* Copy agent data to #1 variables */
	x1 = get_posx();
	y1 = get_posy();
	z1 = get_posz();
	s1 = get_state();
	
	/* Interaction radius of the agent (squared so don't need to sqrt compared variable) */
	closestdist = get_iradius()*get_iradius();
	/* Index of closest agent to interact with and set to default as own id */
	closestid = get_id();

	// Check for possible bindings /
	// Get first location message /
	Erklocation_message = get_first_Erklocation_message();
	// And loop through them until there are none left /
	
	while(Erklocation_message) 
	{
		// If the location message is not from the current agent /
		if((Erklocation_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = Erklocation_message->id;
			x2 = Erklocation_message->x;
			y2 = Erklocation_message->y;
			z2 = Erklocation_message->z;
			s2 = Erklocation_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				if(
				// if agent MEK is free in cyto and active - aMEK, and, Erk is dormant free in cytoplasm - Erk /
				(s1 == 42 && s2 == 50) 				
				)

				{
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		// Get next location message /
		Erklocation_message = get_next_Erklocation_message(Erklocation_message);
	}	
	
	// If closest id is not the sham value therefore a possible bond has been found /
	if(closestid != xmemory->id)
	{
		// Send try to bond message /
		add_MEKnewbond_message(xmemory->id,         
						xmemory->state,
						closestid,           // id of agent message is refering to /
						3,                   // 3 is for a try to bond /
						closestdist,        // distance to between agents /
						get_iradius(),
						get_posx(),
						get_posy(),
						get_posz());
	}
	
	return 0;
}

int MEK_checkbondtries()
{
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_MEK * xmemory = current_xmachine->xmachine_MEK;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	double xRaf, yRaf, zRaf;
	
	/* Check if I have been bound to Raf */
	/* Get first bond message from Raf agent*/
	Rafnewbond_message = get_first_Rafnewbond_message();
	/* And loop through messages until there are none left */
	while(Rafnewbond_message)
	{
		if(Rafnewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (Rafnewbond_message->distance < closestdist))
			{
				closestdist = Rafnewbond_message->distance;
				closestid = Rafnewbond_message->idfrom;
				closeststate = Rafnewbond_message->statefrom;
				xRaf = Rafnewbond_message->x;
				yRaf = Rafnewbond_message->y;
				zRaf = Rafnewbond_message->z;
			}
		}
		
		/* Get next trybond message */
		Rafnewbond_message = get_next_Rafnewbond_message(Rafnewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						/* If MEK free in cyto and dormant and Raf free in cyto and active, set MEK to abMEK and Raf to ab2Raf  
						Raf has moved to Integrin and come in contact --- movement is instantaneous
						Also set ACS10 to 10 -- aritrary  --*/
						if(get_state() == 40  && closeststate == 32) 
						{
							set_state(41);											
							set_ACS10(10);
							
	                    /*  Assuming instantaneous movement--repositioning of MEK so that it is in contact with Raf that its bound to */		
							set_posx(xRaf); 
							set_posy(yRaf); 
							set_posz(zRaf);
						}
						/* Send bond message to molecule to bind */
		add_MEKfinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          /* id of Raf agent message is refering to */
						3,
						closestdist,    /* distance previously not used/ now used for try */
						get_iradius(),
						xRaf,
						yRaf,
						zRaf);

	}

	return 0;
}

int MEK_move()
{
	xmachine_memory_MEK * xmemory = current_xmachine->xmachine_MEK;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving dormant or active MEK in cytoplam*/
	if(( xmemory->state == 40 || xmemory->state == 42 ))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
		/* If MEK active and bound to Raf or MEK*/
		if((xmemory->state == 41 || xmemory->state == 43))
		{
			//stay in position bound to agent/
		}
		
	/* Check if I have been bound to Erk */
	/* Get first bond message */
	Erkfinalbond_message = get_first_Erkfinalbond_message();
	/* And loop through messages until there are none left */
	while(Erkfinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(Erkfinalbond_message->idto == xmemory->id )
		{

		/* If agent MEK is aMEK(42), Change state to ab2MEK(43)*/
		if(get_state()== 42 ) 
						{
						set_state(43);
						set_ACS11(8); //literature
						}
		}
		/* Get next bond message */
		Erkfinalbond_message = get_next_Erkfinalbond_message(Erkfinalbond_message);			
	}
				
	
	return 0;
}	

int Erk_outputdata()
{
		/* Send location message */
	add_Erklocation_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state(),	/* agent type and state */
						get_iradius());	/* agent message range */
						
		/* Send availability message */
	add_Erkavailability_message(get_id(),		/* agent id */
						get_state(),		/* agent type and state */
						get_compartmentid());	/* whether in nucleus or cytoplasm */
						
	/* if Erk is dormant - free in cytoplasm*/
	
	/* if Erk is bound to MEK and active - abErk(51) */
	if(get_state() == 51)
	{
		if(get_ACS13() > 0)
		{
			/* Then decrement ACS13 counter */
			set_ACS13(get_ACS13() - 1);
			/* If processing has finished (ACS13 is zero) */
			if(get_ACS13() == 0)
			{
			/* Switch to active unbound Erk free in cyto  and set ACS14 to 50 - from half life value of 50s from (Fujioka 2006)*/
				set_state(52);
				set_ACS14(50);	
				GLOBAL_cyto_pErk_count++;
			}
		}
		/* ACS13 will only reset when a bond between MEK and Erk has been formed*/
	}	
	/* if Erk is dormant - free in cytoplasm*/
	
		
	/* if Erk is active and free in cyto - aErk(52) */
	if(get_state() == 52)
	{
		//randomly bind some of the active Erks in the nucleus to molecular complexes to simulate the lower availability of Erk in the nucleus due toits binding to other molecules/
		if(get_compartmentid() == 1)
		{
			if((srand()/(double)(RAND_MAX)*10) < 2)
			{
				//switch to ab3Erk (54) - bound to molecular complexes/
				set_state(54);
				set_ACS16(6);
			}
		}
		if(get_ACS14() > 0)
		{
			/* Then decrement ACS14 counter */
			set_ACS14(get_ACS14() - 1);
			/* If processing has finished (ACS14 is zero) */
			if(get_ACS14() == 0)
			{
			/* Switch to dormant Erk free in cyto Erk(50)*/
				set_state(50);
					if(get_compartmentid() == 1)
					{
						
					}
			}
		}
		/* ACS14 will only reset when active Erk dissociates from MEK*/
	}	
	
	/* if Erk is bound to Runx2 and active - ab2Erk(53) */
	if(get_state() == 53)
	{
		if(get_ACS15() > 0)
		{
			/* Then decrement ACS15 counter */
			set_ACS15(get_ACS15() - 1);
			/* If processing has finished (ACS15 is zero) */
			if(get_ACS15() == 0)
			{
			/* Switch to active unbound erk free in nucleo  and set ACS5 to previous ACS6  */
				set_state(52);	
			}
		}
		/* ACS15 will only reset when a bond between Erk and Runx2 has been formed*/
	}	
	
	/* if Erk is bound to other molecular complexes and active - ab3Erk(54) */
	if(get_state() == 54)
	{
		if(get_ACS16() > 0)
		{
			/* Then decrement ACS16 counter */
			set_ACS16(get_ACS16() - 1);
			/* If processing has finished (ACS16 is zero) */
			if(get_ACS16() == 0)
			{
			/* Switch to active unbound Erk free in nucleo  and set ACS5 to previous ACS6  */
				set_state(52);	
			}
		}
		/* ACS16 will only reset when a bond between Erk and some molecular complex has been formed*/
	}	
	return 0;
}	

int Erk_inputdata()
{
	xmachine_memory_Erk * xmemory = current_xmachine->xmachine_Erk;
	/* Create variables to use */
	double x1, y1, x2, y2, z1, z2, closestdist;
	int id2, s1, s2, closestid, closeststate; 
	double distance;
		
	/* Copy agent data to #1 variables */
	x1 = get_posx();
	y1 = get_posy();
	z1 = get_posz();
	s1 = get_state();
	
	/* Interaction radius of the agent (squared so don't need to sqrt compared variable) */
	closestdist = get_iradius()*get_iradius();
	/* Index of closest agent to interact with and set to default as own id */
	closestid = get_id();
	
	// Check for possible bindings /
	// Get first location message /
	Runx2location_message = get_first_Runx2location_message();
	// And loop through them until there are none left /
	
	while(Runx2location_message) 
	{
		// If the location message is not from the current agent /
		if((Runx2location_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = Runx2location_message->id;
			x2 = Runx2location_message->x;
			y2 = Runx2location_message->y;
			z2 = Runx2location_message->z;
			s2 = Runx2location_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				if(
				// if agent Erk is free in nucleoplasm and active - aErk, and, Runx2 is dormant free in nucleoplasm - Runx2 /
				(s1 == 52 && s2 == 60) 				
				)

				{
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		// Get next location message /
		Runx2location_message = get_next_Runx2location_message(Runx2location_message);
	}	
	
	// If closest id is not the sham value therefore a possible bond has been found /
	if(closestid != xmemory->id)
	{
		// Send try to bond message /
		add_Erknewbond_message(xmemory->id,         
						xmemory->state,
						closestid,           // id of agent message is refering to /
						3,                   // 3 is for a try to bond /
						closestdist,        // distance to between agents /
						get_iradius(),
						get_posx(),
						get_posy(),
						get_posz());
	}
	
	return 0;
}

int Erk_checkbondtries()
{
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_Erk * xmemory = current_xmachine->xmachine_Erk;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	double xMEK, yMEK, zMEK;
	
	/* Check if I have been bound to MEK */
	/* Get first bond message from MEK agent*/
	MEKnewbond_message = get_first_MEKnewbond_message();
	/* And loop through messages until there are none left */
	while(MEKnewbond_message)
	{
		if(MEKnewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (MEKnewbond_message->distance < closestdist))
			{
				closestdist = MEKnewbond_message->distance;
				closestid = MEKnewbond_message->idfrom;
				closeststate = MEKnewbond_message->statefrom;
				xMEK = MEKnewbond_message->x;
				yMEK = MEKnewbond_message->y;
				zMEK = MEKnewbond_message->z;
			}
		}
		
		/* Get next trybond message */
		MEKnewbond_message = get_next_MEKnewbond_message(MEKnewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						/* If Erk free in cyto and dormant and MEK free in cyto and active, set Erk to abErk and MEK to ab2MEK  
						MEK has moved to Integrin and come in contact --- movement is instantaneous
						Also set ACS13 to 5 -- aritrary  --*/
						if(get_state() == 50  && closeststate == 42) 
						{
							set_state(51);											
							set_ACS13(8);
							
	                    /*  Assuming instantaneous movement--repositioning of Erk so that it is in contact with MEK that its bound to */		
							set_posx(xMEK); 
							set_posy(yMEK); 
							set_posz(zMEK);
						}
						/* Send bond message to molecule to bind */
		add_Erkfinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          /* id of MEK agent message is refering to */
						3,
						closestdist,    /* distance previously not used/ now used for try */
						get_iradius(),
						xMEK,
						yMEK,
						zMEK);

	}

	return 0;
}

int Erk_move()
{
	xmachine_memory_Erk * xmemory = current_xmachine->xmachine_Erk;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving dormant Erk in cytoplam*/
	if(( xmemory->state == 50) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
		/* If Erk active and bound to MEK*/
		if((xmemory->state == 51))
		{
			//stay in position bound to integrin/
		}
			
		/* If Erk active and bound to Runx2 or other molecular complexes*/
		if((xmemory->state == 53) || (xmemory->state == 54))
		{
			//stay in position bound to Runx2 or other molecular complexes/
		}
			
	/* If freely moving active Erk in cytoplam*/
	if(( xmemory->state == 52 ) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is within translocateable distance from nucleus */
		if(xmemory->posr < (nuclradius + 150) )
		{	
				srand ( time(NULL) );
				int randomnum = (srand() % 3);
			if((randomnum == 0)||(randomnum == 1))
			{
				/* Translocate into the nucleus */
				xmemory->posr =(xmemory->posr - 160);
				// update compartmentid 0 = cyto, 1 = nucleus/
				set_compartmentid(1);
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement
				movex = -movex;
				movey = -movey;
				movez = -movez;*/
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			if(randomnum == 2)
			{
				/* Mirror position back into cell */
				xmemory->posr = (nuclradius + 160);
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
		}
	}
	/* If freely moving active Erk in nucleus*/
	if(( xmemory->state == 52 ) && (xmemory->compartmentid == 1))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the nuclear membrane */
			if(xmemory->posr > nuclradius)
			{
				/* Mirror position back into nucleus */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
	
	// If freely moving dormant Erk in nucleus
	if(( xmemory->state == 50 ) && (xmemory->compartmentid == 1)) 
		{
		// Calculate and update movement in polar coordinates /

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		// Calculate corresponding movement in cartesian coordinates /
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		// Update cartesian positions with respect to time /
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		// Calculate polar position from cartesian position /
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		// If position is within translocateable distance from nucleus /
			if(xmemory->posr > (nuclradius - 50))
			{
				// Translocate out of nucleus into cytoplasm /
				xmemory->posr = (xmemory->posr + 60);
				// update compartmentid 0 = cyto, 1 = nucleus/
				set_compartmentid(0);
				// Update new position /
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement /
				movex = -movex;
				movey = -movey;
				movez = -movez;*/
				// Update new movement /
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		/* Check if I have been bound to Runx2 */
	/* Get first bond message */
	Runx2finalbond_message = get_first_Runx2finalbond_message();
	/* And loop through messages until there are none left */
	while(Runx2finalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(Runx2finalbond_message->idto == xmemory->id )
		{

		/* If agent Erk is aErk(52), Change state to ab2Erk(53)*/
		if(get_state()== 52 ) 
						{
						set_state(53);
						set_ACS15(10); //literature based
						}
		}
		/* Get next bond message */
		Runx2finalbond_message = get_next_Runx2finalbond_message(Runx2finalbond_message);			
	}
	
	return 0;
}	

int Runx2_outputdata()
{
		/* Send location message */
	add_Runx2location_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state(),	/* agent type and state */
						get_iradius());	/* agent message range */
						
	/* if Runx2 is dormant - free in cytoplasm*/
	
	/* if Runx2 is bound to Erk and active - abRunx2(61) */
	if(get_state() == 61)
	{
		if(get_ACS17() > 0)
		{
			/* Then decrement ACS17 counter */
			set_ACS17(get_ACS17() - 1);
			/* If processing has finished (ACS17 is zero) */
			if(get_ACS17() == 0)
			{
			/* Switch to active unbound Runx2 free in cyto  and set ACS19 to 60*/
				set_state(62);
				set_ACS19(60);	
				//pRunx2_count++;
			}
		}
		/* ACS17 will only reset when a bond between Erk and Runx2 has been formed*/
	}	
	/* if Runx2 is dormant - free in cytoplasm*/
	
		
	/* if Runx2 is active and free in cyto - aRunx2(62) */
	if(get_state() == 62)
	{
		
		if(get_ACS19() > 0)
		{
			/* Then decrement ACS19 counter */
			set_ACS19(get_ACS19() - 1);
			/* If processing has finished (ACS19 is zero) */
			if(get_ACS19() == 0)
			{
			/* Switch to dormant Runx2 free in cyto Runx2(60)*/
				set_state(60);
					if(get_compartmentid() == 1)
					{
						
					}
			}
		}
		/* ACS19 will only reset when active Runx2 dissociates from Erk*/
	}	
	
	/* if Runx2 is bound to Complex and active - ab2Runx2(63) */
	if(get_state() == 63)
	{
		if(get_ACS18() > 0)
		{
			/* Then decrement ACS18 counter */
			set_ACS18(get_ACS18() - 1);
			/* If processing has finished (ACS18 is zero) */
			if(get_ACS18() == 0)
			{
			/* Switch to active unbound Runx2 free in nucleo  //and set ACS5 to previous ACS6  */
				set_state(62);	
			}
		}
		/* ACS18 will only reset when a bond between Runx2 and Complex has been formed*/
	}	
	
		
	return 0;
}	

int Runx2_inputdata()
{
	xmachine_memory_Runx2 * xmemory = current_xmachine->xmachine_Runx2;
	/* Create variables to use */
	double x1, y1, x2, y2, z1, z2, closestdist;
	int id2, s1, s2, closestid, closeststate; 
	double distance;
		
	/* Copy agent data to #1 variables */
	x1 = get_posx();
	y1 = get_posy();
	z1 = get_posz();
	s1 = get_state();
	
	/* Interaction radius of the agent (squared so don't need to sqrt compared variable) */
	closestdist = get_iradius()*get_iradius();
	/* Index of closest agent to interact with and set to default as own id */
	closestid = get_id();
	
	// Check for possible bindings /
	// Get first location message /
	Complexlocation_message = get_first_Complexlocation_message();
	// And loop through them until there are none left /
	
	while(Complexlocation_message) 
	{
		// If the location message is not from the current agent /
		if((Complexlocation_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = Complexlocation_message->id;
			x2 = Complexlocation_message->x;
			y2 = Complexlocation_message->y;
			z2 = Complexlocation_message->z;
			s2 = Complexlocation_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				
				// if agent Runx2 is free in nucleoplasm and active - aRunx2 /
				if((s1 == 62)) 	
				{
					//And if complex is free in nucleoplasm - in any of its states(mono, multi,DNA bound etc)/
					if((s2 == 70) || (s2 == 71) || (s2 == 72) || (s2 == 73))
				  {
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				  }
				}
			}
		}
		// Get next location message /
		Complexlocation_message = get_next_Complexlocation_message(Complexlocation_message);
	}	
	
	// If closest id is not the sham value therefore a possible bond has been found /
	if(closestid != xmemory->id)
	{
		// Send try to bond message /
		add_Runx2newbond_message(xmemory->id,         
						xmemory->state,
						closestid,           // id of agent message is refering to /
						closeststate,        // state of the complex agent message is refering to/
						closestdist,        // distance to between agents /
						get_iradius(),
						get_posx(),
						get_posy(),
						get_posz());
	}
	
	return 0;
}

int Runx2_checkbondtries()
{
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_Runx2 * xmemory = current_xmachine->xmachine_Runx2;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	double xErk, yErk, zErk;
	
	/* Check if I have been bound to Erk */
	/* Get first bond message from Erk agent*/
	Erknewbond_message = get_first_Erknewbond_message();
	/* And loop through messages until there are none left */
	while(Erknewbond_message)
	{
		if(Erknewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (Erknewbond_message->distance < closestdist))
			{
				closestdist = Erknewbond_message->distance;
				closestid = Erknewbond_message->idfrom;
				closeststate = Erknewbond_message->statefrom;
				xErk = Erknewbond_message->x;
				yErk = Erknewbond_message->y;
				zErk = Erknewbond_message->z;
			}
		}
		
		/* Get next trybond message */
		Erknewbond_message = get_next_Erknewbond_message(Erknewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						/* If Runx2 free in cyto and dormant and Erk free in cyto and active, set Runx2 to abRunx2 and Erk to ab2Erk  
						Erk has moved to Integrin and come in contact --- movement is instantaneous
						Also set ACS13 to 5 -- aritrary  --*/
						if(get_state() == 60  && closeststate == 52) 
						{
							set_state(61);											
							set_ACS17(10);
							
	                    /*  Assuming instantaneous movement--repositioning of Runx2 so that it is in contact with Erk that its bound to */		
							set_posx(xErk); 
							set_posy(yErk); 
							set_posz(zErk);
						}
						/* Send bond message to molecule to bind */
		add_Runx2finalbond_message(xmemory->id,
						xmemory->state,
						closestid,          /* id of Erk agent message is refering to */
						3,
						closestdist,    /* distance previously not used/ now used for try */
						get_iradius(),
						xErk,
						yErk,
						zErk);

	}

	return 0;
}

int Runx2_move()
{
	xmachine_memory_Runx2 * xmemory = current_xmachine->xmachine_Runx2;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;



		/* If Runx2 active and bound to ERK*/
		if((xmemory->state == 61))
		{
			//stay in position bound to ERK/
		}
			
		// If Runx2 active and bound to Complex /
		if((xmemory->state == 63))
		{
			//stay in position bound to Complex/
		}
			
	
	/* If freely moving active or dormant Runx2 in nucleus*/
	if(( xmemory->state == 62 ) || ( xmemory->state == 60 ))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the nuclear membrane */
			if(xmemory->posr > nuclradius)
			{
				/* Mirror position back into nucleus */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		
		/* Check if I have been bound to Complex */
	/* Get first bond message */
	Complexfinalbond_message = get_first_Complexfinalbond_message();
	/* And loop through messages until there are none left */
	while(Complexfinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(Complexfinalbond_message->idto == xmemory->id )
		{

		/* If agent Runx2 is aRunx2(62), Change state to ab2Runx2(63)*/
		if(get_state()== 62 ) 
						{
						set_state(63);
						set_ACS18(10); //arbitrary
						}
		}
		/* Get next bond message */
		Complexfinalbond_message = get_next_Complexfinalbond_message(Complexfinalbond_message);			
	}
	
	return 0;
}	

int Complex_outputdata()
{
		/* Send location message */
	add_Complexlocation_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state(),	/* agent type and state */
						get_iradius());	/* agent message range */
						
	
	
	/* if Complex is bound to Runx2  */
	if((get_state() == 700) || (get_state() == 710) || (get_state() == 720) || (get_state() == 730) || (get_state() == 711))
	{
		if(get_ACS20() > 0)
		{
			/* Then decrement ACS20 counter */
			set_ACS20(get_ACS20() - 1);
			/* If processing has finished (ACS20 is zero) */
			if(get_ACS20() == 0)
			{
			// Switch to unbound Complex free in nucleo 
				set_state(70);	
			}
		}
		/* ACS20 will only reset when a bond between Complex and Runx2 has been formed*/
		if(get_ACS21() > 0)
		{
			/* Then decrement ACS21 counter */
			set_ACS21(get_ACS21() - 1);
			/* If processing has finished (ACS21 is zero) */
			if(get_ACS21() == 0)
			{
			// Switch to unbound Complex free in nucleo 
				set_state(71);	
			}
		}
		/* ACS21 will only reset when a bond between Complex and Runx2 has been formed*/
		if(get_ACS22() > 0)
		{
			/* Then decrement ACS22 counter */
			set_ACS22(get_ACS22() - 1);
			/* If processing has finished (ACS22 is zero) */
			if(get_ACS22() == 0)
			{
			// Switch to unbound Complex free in nucleo 
				set_state(71);	
			}
		}
		/* ACS22 will only reset when a bond between Complex and Runx2 has been formed*/
		if(get_ACS23() > 0)
		{
			/* Then decrement ACS23 counter */
			set_ACS23(get_ACS23() - 1);
			/* If processing has finished (ACS23 is zero) */
			if(get_ACS23() == 0)
			{
			// Switch to unbound Complex free in nucleo 
				set_state(72);	
			}
		}
		/* ACS23 will only reset when a bond between Complex and Runx2 has been formed*/
		if(get_ACS24() > 0)
		{
			/* Then decrement ACS24 counter */
			set_ACS24(get_ACS24() - 1);
			/* If processing has finished (ACS24 is zero) */
			if(get_ACS24() == 0)
			{
			// Switch to unbound Complex free in nucleo 
				set_state(73);	
			}
		}
		/* ACS24 will only reset when a bond between Complex and Runx2 has been formed*/
	}	
	
		
	return 0;
}	

int Complex_transcribe()
{
	srand ( time(NULL) );
	xmachine_memory_Complex * xmemory = current_xmachine->xmachine_Complex;
	int productid = get_productid(); //use this to solve issues with product ids
	
	// if the multimeric complex with Osx, Tfs and DNA is bound to Runx2 - forms an actively transcribing complex (730)/
	if((get_state() == 730) || get_state() == 73)
	{
		if(get_OCNdelay() > 0)
		{
			/* Then decrement OCNdelay counter */
			set_OCNdelay(get_OCNdelay() - 1);
			/* If processing has finished (OCNdelay is zero) */
			if((get_OCNdelay()%600) == 0) //to allow for a continual process giving complex the ability to restart a transcription process while a previous one going on/
			{
				int disintegrationtime = (2500 +(srand() % 1000));//based on the literature/
			// Add an OCN mRNA agent into the model/
				 add_OCN_agent(productid,
							   get_postheta(),
							   get_posphi(),
							   get_posr(),
							   get_posx(),
							   get_posy(),
							   get_posz(),
							   get_movetheta(),
							   get_movephi(),
							   get_mover(),
							   0,
							   0,
							   100,
							   disintegrationtime,
							   1);	
				productid++;
				if(get_OCNdelay() != 0)
				{
				set_OCNdelay(get_OCNdelay() - get_OCNbalance());	
				}
				
			}
		}
		if(get_OPNdelay() > 0)
		{
			/* Then decrement OPNdelay counter */
			set_OPNdelay(get_OPNdelay() - 1);
			/* If processing has finished (OPNdelay is zero) */
			if((get_OPNdelay()%600) == 0) //to allow for a continual process giving complex the ability to restart a transcription process while a previous one going on/
			{
				int disintegrationtime = (2000 +(srand() % 1000));//based on the literature/
			// Add an OPN mRNA agent into the model/
				 add_OPN_agent(productid,
							   get_postheta(),
							   get_posphi(),
							   get_posr(),
							   get_posx(),
							   get_posy(),
							   get_posz(),
							   get_movetheta(),
							   get_movephi(),
							   get_mover(),
							   0,
							   0,
							   90,
							   disintegrationtime,
							   1);	
				productid++;
				if(get_OPNdelay() != 0)
				{
				set_OPNdelay(get_OPNdelay() - get_OPNbalance());	
				}
				
			}
		}
		if(get_ALPdelay() > 0)
		{
			/* Then decrement ALPdelay counter */
			set_ALPdelay(get_ALPdelay() - 1);
			/* If processing has finished (ALPdelay is zero) */
			if((get_ALPdelay()%600) == 0) //to allow for a continual process giving complex the ability to restart a transcription process while a previous one going on/
			{
				int disintegrationtime = (1800 +(srand() % 1000));//based on the literature/
			// Add an ALP mRNA agent into the model/
				 add_ALP_agent(productid,
							   get_postheta(),
							   get_posphi(),
							   get_posr(),
							   get_posx(),
							   get_posy(),
							   get_posz(),
							   get_movetheta(),
							   get_movephi(),
							   get_mover(),
							   0,
							   0,
							   110,
							   disintegrationtime,
							   1);	
				productid++;
				if(get_ALPdelay() != 0)
				{
				set_ALPdelay(get_ALPdelay() - get_ALPbalance());	
				}
			}
		}
		if(get_BSPdelay() > 0)
		{
			/* Then decrement BSPdelay counter */
			set_BSPdelay(get_BSPdelay() - 1);
			/* If processing has finished (BSPdelay is zero) */
			if((get_BSPdelay()%600) == 0) //to allow for a continual process giving complex the ability to restart a transcription process while a previous one going on/
			{
				int disintegrationtime = (1800 +(srand() % 1000));
			// Add an BSP mRNA agent into the model/
				 add_BSP_agent(productid,
							   get_postheta(),
							   get_posphi(),
							   get_posr(),
							   get_posx(),
							   get_posy(),
							   get_posz(),
							   get_movetheta(),
							   get_movephi(),
							   get_mover(),
							   0,
							   0,
							   120,
							   disintegrationtime,
							   1);	
				productid++;
				if(get_BSPdelay() != 0)
				{
				set_BSPdelay(get_BSPdelay() - get_BSPbalance());	
				}
			}
		}
	}
	
	set_productid(productid);
	
	return 0;
}

int Complex_checkbondtries()
{
	srand ( time(NULL) );
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	int randomnum = (srand() % 4);     // generates random integers 0 to 3/
	
	xmachine_memory_Complex * xmemory = current_xmachine->xmachine_Complex;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	double xRunx2, yRunx2, zRunx2;
	
	/* Check if I have been bound to Runx2 */
	/* Get first bond message from Runx2 agent*/
	Runx2newbond_message = get_first_Runx2newbond_message();
	/* And loop through messages until there are none left */
	while(Runx2newbond_message)
	{
		if(Runx2newbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (Runx2newbond_message->distance < closestdist))
			{
				closestdist = Runx2newbond_message->distance;
				closestid = Runx2newbond_message->idfrom;
				closeststate = Runx2newbond_message->statefrom;
				xRunx2 = Runx2newbond_message->x;
				yRunx2 = Runx2newbond_message->y;
				zRunx2 = Runx2newbond_message->z;
			}
		}
		
		/* Get next trybond message */
		Runx2newbond_message = get_next_Runx2newbond_message(Runx2newbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						// If Complex free in nucleo and dormant and Runx2 free in nucleo and active, set Complex to abComplex and Runx2 to ab2Runx2/
						
						//in the case of monomeric osx binding to Runx2/
						if(get_state() == 70  && closeststate == 62) 
						{
							set_state(700);											
							set_ACS20(20);//arbitrary /
							
	                    /*  Assuming instantaneous movement--repositioning of Complex so that it is in contact with Runx2 that its bound to */		
							set_posx(xRunx2); 
							set_posy(yRunx2); 
							set_posz(zRunx2);
						}
						
						//in the case of monomeric osx + DNA binding to Runx2/
						if(get_state() == 71  && closeststate == 62) 
						{
							set_state(710);											
							set_ACS21(30);//arbitrary /
							
	                    /*  Assuming instantaneous movement--repositioning of Complex so that it is in contact with Runx2 that its bound to */		
							set_posx(xRunx2); 
							set_posy(yRunx2); 
							set_posz(zRunx2);
						}
						
						//in the case of multimeric osx & trascription factors binding to Runx2/
						if(get_state() == 72  && closeststate == 62) 
						{
							set_state(720);											
							set_ACS23(40);//arbitrary  /
							
	                    /*  Assuming instantaneous movement--repositioning of Complex so that it is in contact with Runx2 that its bound to */		
							set_posx(xRunx2); 
							set_posy(yRunx2); 
							set_posz(zRunx2);
						}
						
						//in the case of multimeric osx & trascription factors + DNA binding to Runx2/
						if(get_state() == 73  && closeststate == 62) 
						{
							set_state(730);											
							set_ACS24(50);//arbitrary  , complex can reinitiate transcription afetr this time/
							if (randomnum == 0)
							{
								if(get_OCNdelay() > 0)
								{
									set_OCNbalance((get_OCNdelay()%600));
								}
								set_OCNdelay(get_OCNdelay() + 600);//arbitrary value/
							}
							if (randomnum == 1)
							{
								if(get_OPNdelay() > 0)
								{
									set_OPNbalance((get_OPNdelay()%600));
								}
								set_OPNdelay(get_OPNdelay() + 600);//arbitrary value/
							}
							if (randomnum == 2)
							{
								if(get_ALPdelay() > 0)
								{
									set_ALPbalance((get_ALPdelay()%600));
								}
								set_ALPdelay(get_ALPdelay() + 600);//arbitrary value/
							}
							if (randomnum == 3)
							{
								if(get_BSPdelay() > 0)
								{
									set_BSPbalance((get_BSPdelay()%600));
								}
								set_BSPdelay(get_BSPdelay() + 600);//arbitrary value/
							}
	                    /*  Assuming instantaneous movement--repositioning of Complex so that it is in contact with Runx2 that its bound to */		
							set_posx(xRunx2); 
							set_posy(yRunx2); 
							set_posz(zRunx2);
						}
						/* Send bond message to molecule to bind */
						
		add_Complexfinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          /* id of Runx2 agent message is refering to */
						3,
						closestdist,    /* distance previously not used/ now used for try */
						get_iradius(),
						xRunx2,
						yRunx2,
						zRunx2);

	}

	return 0;
}

int Complex_move()
{
	xmachine_memory_Complex * xmemory = current_xmachine->xmachine_Complex;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double DNAmoveboundary = 300.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 1;         /* Average speed of Complex agent <assumed lower than speed of protein agents>*/
	double speed_range = 0.5;       /* Range of speed, + or - from average speed <assumed lower than range of protein agents>*/
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


// we assume that the DNA and TF complex doesn't move once bound to Runx2 just to differentiate from available Complex/
		/* If Complex active and bound to Runx2*/
		if((xmemory->state == 700) || (xmemory->state == 710) || (xmemory->state == 720) || (xmemory->state == 730) || (xmemory->state == 711))
		{
			//stay in position bound to Runx2/
		}
		
			
// we assume that the DNA and TF complex moves if not bound to Runx2 just to differentiate from Complex bound to Runx2/
	/* If freely moving Complex in nucleus not bound to Runx2*/
		if((xmemory->state == 70) || (xmemory->state == 71) || (xmemory->state == 72) || (xmemory->state == 73))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the DNAboundary */
			if(xmemory->posr > DNAmoveboundary)
			{
				/* Mirror position back into DNAboundary */
				xmemory->posr = (2*DNAmoveboundary)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		
	
	
	return 0;
}	

int OCN_outputdata()
{
		/* Send location message */
	add_OCNlocation_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state());	/* agent type and state */
						
	/* if OCN is unbound free in cytoplasm*/
	
	/* if OCNmRNA is bound to Ribosome and in unavailable state - OCN(102) */
	if(get_state() == 102)
	{
		if(get_ACS27() > 0)
		{
			
			/* Then decrement ACS27 counter */
			set_ACS27(get_ACS27() - 1);
			/* If processing has finished (ACS27 is zero) */
			if(get_ACS27() == 0)
			{
			/* Switch to bound available OCN mRNA and set ACS28 to 70  ( 75 +/- 20 is typical time for translation - based on literature)*/
				set_state(101);
				set_ACS28(75);		
			}
		}
		/* ACS27 will only reset when a bond between Ribosome and OCN has been formed*/
	}

		/* if OCNmRNA is bound to Ribosome and in available state - OCN(101) */
	if(get_state() == 101)
	{
		if(get_ACS28() > 0)
		{
			/* Then decrement ACS28 counter */
			set_ACS28(get_ACS28() - 1);
			/* If processing has finished (ACS28 is zero) */
			if(get_ACS28() == 0)
			{
			/* Switch to unbound OCN mRNA */
				set_state(100);		
			}
		}
		/* ACS28 will only reset when OCN switches from unavailable to available bound*/
	}
	
		/* if mRNA exists and disintegration time is > 0 */
	if(get_disintegrationtime() > 0)
	{
		
			/* Then decrement disintegrationtime counter */
			set_disintegrationtime(get_disintegrationtime() - 1);
			/* If processing has finished (disintegrationtime is zero) */
			if(get_disintegrationtime() == 0)
			{
			/* remove mRNA agent from model, i.e death of particular mRNA agent */
				return 1;		
			}
		/* Disintegration time resets only when mRNA agent is produced*/
	}
	
	return 0;
}	

int OCN_checkbondtries()
{
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_OCN * xmemory = current_xmachine->xmachine_OCN;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	
	/* Check if I have been bound to Ribosome */
	/* Get first bond message from Ribosome agent*/
	Ribosomenewbond_message = get_first_Ribosomenewbond_message();
	/* And loop through messages until there are none left */
	while(Ribosomenewbond_message)
	{
		if(Ribosomenewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (Ribosomenewbond_message->distance < closestdist))
			{
				closestdist = Ribosomenewbond_message->distance;
				closestid = Ribosomenewbond_message->idfrom;
				closeststate = Ribosomenewbond_message->statefrom;
			}
		}
		
		/* Get next trybond message */
		Ribosomenewbond_message = get_next_Ribosomenewbond_message(Ribosomenewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						// If OCN mRNA is available to bind and if Ribosome is available to bind  
						if(((get_state() == 100)||(get_state() == 101))  && ((closeststate == 130)||(closeststate == 140))) 
						{
							set_state(102);											
							set_ACS27(20);
						}
						/* Send bond message to molecule to bind */
		add_OCNfinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          // id of Ribosome agent that is involved in this particular bind/
						closeststate,       //state of Ribosome agent that is involved in this particular bind/
						get_posx(),
						get_posy(),
						get_posz());

	}

	return 0;
}

int OCN_move()
{
	xmachine_memory_OCN * xmemory = current_xmachine->xmachine_OCN;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent -- Adapted from literature...*/
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving OCN mRNA in cytoplam*/
	if(( xmemory->state == 100) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
	
	// If freely moving OCN mRNA in nucleus
	if(( xmemory->state == 100 ) && (xmemory->compartmentid == 1)) 
		{
		// Calculate and update movement in polar coordinates /

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		// Calculate corresponding movement in cartesian coordinates /
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		// Update cartesian positions with respect to time /
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		// Calculate polar position from cartesian position /
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		// If position is within translocateable distance from nucleus /
			if(xmemory->posr > (nuclradius - 50))
			{
				// Translocate out of nucleus into cytoplasm /
				xmemory->posr = (xmemory->posr + 60);
				// update compartmentid 0 = cyto, 1 = nucleus/
				set_compartmentid(0);
				// Update new position /
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement /
				movex = -movex;
				movey = -movey;
				movez = -movez;*/
				// Update new movement /
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
	
	
	/* If OCN mRNA bound to Ribosome*/
	if(( xmemory->state == 101 ) || (xmemory->state == 102))
	{
		
	}		
		
	
	return 0;
}	

int OPN_outputdata()
{
		/* Send location message */
	add_OPNlocation_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state());	/* agent type and state */
						
	/* if OPN is unbound free in cytoplasm*/
	
	/* if OPNmRNA is bound to Ribosome and in unavailable state - OPN(92) */
	if(get_state() == 92)
	{
		if(get_ACS25() > 0)
		{
			/* Then decrement ACS25 counter */
			set_ACS25(get_ACS25() - 1);
			/* If processing has finished (ACS25 is zero) */
			if(get_ACS25() == 0)
			{
			/* Switch to bound available OPN mRNA and set ACS26 to 90 (typical time for translation - literature based)*/
				set_state(91);
				set_ACS26(75);		
			}
		}
		/* ACS25 will only reset when a bond between Ribosome and OPN has been formed*/
	}

		/* if OPNmRNA is bound to Ribosome and in available state - OPN(91) */
	if(get_state() == 91)
	{
		if(get_ACS26() > 0)
		{
			/* Then decrement ACS26 counter */
			set_ACS26(get_ACS26() - 1);
			/* If processing has finished (ACS26 is zero) */
			if(get_ACS26() == 0)
			{
			/* Switch to unbound OPN mRNA */
				set_state(90);		
			}
		}
		/* ACS26 will only reset when OPN switches from unavailable to available bound*/
	}
	
	/* if mRNA exists and disintegration time is > 0 */
	if(get_disintegrationtime() > 0)
	{
		
			/* Then decrement disintegrationtime counter */
			set_disintegrationtime(get_disintegrationtime() - 1);
			/* If processing has finished (disintegrationtime is zero) */
			if(get_disintegrationtime() == 0)
			{
			/* remove mRNA agent from model, i.e death of particular mRNA agent */
				return 1;		
			}
		/* Disintegration time resets only when mRNA agent is produced*/
	}
	
	return 0;
}	

int OPN_checkbondtries()
{
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_OPN * xmemory = current_xmachine->xmachine_OPN;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	
	/* Check if I have been bound to Ribosome */
	/* Get first bond message from Ribosome agent*/
	Ribosomenewbond_message = get_first_Ribosomenewbond_message();
	/* And loop through messages until there are none left */
	while(Ribosomenewbond_message)
	{
		if(Ribosomenewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (Ribosomenewbond_message->distance < closestdist))
			{
				closestdist = Ribosomenewbond_message->distance;
				closestid = Ribosomenewbond_message->idfrom;
				closeststate = Ribosomenewbond_message->statefrom;
			}
		}
		
		/* Get next trybond message */
		Ribosomenewbond_message = get_next_Ribosomenewbond_message(Ribosomenewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						// If OPN mRNA is available to bind and if Ribosome is available to bind  
						if(((get_state() == 90)||(get_state() == 91))  && ((closeststate == 130)||(closeststate == 140))) 
						{
							set_state(92);											
							set_ACS25(20);
						}
						/* Send bond message to molecule to bind */
		add_OPNfinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          // id of Ribosome agent that is involved in this particular bind/
						closeststate,       //state of Ribosome agent that is involved in this particular bind/
						get_posx(),
						get_posy(),
						get_posz());

	}

	return 0;
}

int OPN_move()
{
	xmachine_memory_OPN * xmemory = current_xmachine->xmachine_OPN;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving OPN mRNA in cytoplam*/
	if(( xmemory->state == 90) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
	
	// If freely moving OPN mRNA in nucleus
	if(( xmemory->state == 90 ) && (xmemory->compartmentid == 1)) 
		{
		// Calculate and update movement in polar coordinates /

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		// Calculate corresponding movement in cartesian coordinates /
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		// Update cartesian positions with respect to time /
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		// Calculate polar position from cartesian position /
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		// If position is within translocateable distance from nucleus /
			if(xmemory->posr > (nuclradius - 50))
			{
				// Translocate out of nucleus into cytoplasm /
				xmemory->posr = (xmemory->posr + 60);
				// update compartmentid 0 = cyto, 1 = nucleus/
				set_compartmentid(0);
				// Update new position /
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement /
				movex = -movex;
				movey = -movey;
				movez = -movez;*/
				// Update new movement /
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		
		/* If OPN mRNA bound to Ribosome*/
	if(( xmemory->state == 91 ) || (xmemory->state == 92))
	{
		//stay in position/
	}		
	
	return 0;
}	


int ALP_outputdata()
{
	

		// Send location message /
	add_ALPlocation_message(get_id(),		// agent id /
						get_posx(),		// agent x-axis position /
						get_posy(),		// agent y-axis position /
						get_posz(),		// agent z-axis position /
						get_state());	//agent type and state /
						
	/* if ALP is unbound free in cytoplasm*/
	
	/* if ALPmRNA is bound to Ribosome and in unavailable state - ALP(112) */
	if(get_state() == 112)
	{
		if(get_ACS29() > 0)
		{
			/* Then decrement ACS29 counter */
			set_ACS29(get_ACS29() - 1);
			/* If processing has finished (ACS29 is zero) */
			if(get_ACS29() == 0)
			{
			/* Switch to bound available ALP mRNA and set ACS30 to 110 (typical time for translation - can be changed later acc to length of protein product)*/
				set_state(111);
				set_ACS30(75);		
			}
		}
		/* ACS29 will only reset when a bond between Ribosome and ALP has been formed*/
	}

		/* if ALPmRNA is bound to Ribosome and in available state - ALP(111) */
	if(get_state() == 111)
	{
		if(get_ACS30() > 0)
		{
			/* Then decrement ACS30 counter */
			set_ACS30(get_ACS30() - 1);
			/* If processing has finished (ACS30 is zero) */
			if(get_ACS30() == 0)
			{
			/* Switch to unbound ALP mRNA */
				set_state(110);		
			}
		}
		/* ACS30 will only reset when ALP switches from unavailable to available bound*/
	}
	
	/* if mRNA exists and disintegration time is > 0 */
	if(get_disintegrationtime() > 0)
	{
		
			/* Then decrement disintegrationtime counter */
			set_disintegrationtime(get_disintegrationtime() - 1);
			/* If processing has finished (disintegrationtime is zero) */
			if(get_disintegrationtime() == 0)
			{
			/* remove mRNA agent from model, i.e death of particular mRNA agent */
				return 1;		
			}
		/* Disintegration time resets only when mRNA agent is produced*/
	}
	
 

	return 0;
}	

int ALP_checkbondtries()
{

	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_ALP * xmemory = current_xmachine->xmachine_ALP;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	
	/* Check if I have been bound to Ribosome */
	/* Get first bond message from Ribosome agent*/
	Ribosomenewbond_message = get_first_Ribosomenewbond_message();
	/* And loop through messages until there are none left */
	while(Ribosomenewbond_message)
	{
		if(Ribosomenewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (Ribosomenewbond_message->distance < closestdist))
			{
			
				closestdist = Ribosomenewbond_message->distance;
				closestid = Ribosomenewbond_message->idfrom;
				closeststate = Ribosomenewbond_message->statefrom;
			
			}
		}
		
		/* Get next trybond message */
		Ribosomenewbond_message = get_next_Ribosomenewbond_message(Ribosomenewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						// If ALP mRNA is available to bind and if Ribosome is available to bind  
						if(((get_state() == 110)||(get_state() == 111))  && ((closeststate == 130)||(closeststate == 140))) 
						{
							set_state(112);											
							set_ACS29(20);
						}
						/* Send bond message to molecule to bind */
		add_ALPfinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          // id of Ribosome agent that is involved in this particular bind/
						closeststate,       //state of Ribosome agent that is involved in this particular bind/
						get_posx(),
						get_posy(),
						get_posz());

	}
	

	return 0;
}

int ALP_move()
{
 
	xmachine_memory_ALP * xmemory = current_xmachine->xmachine_ALP;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving ALP mRNA in cytoplam*/
	if(( xmemory->state == 110) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
	
	// If freely moving ALP mRNA in nucleus
	if(( xmemory->state == 110 ) && (xmemory->compartmentid == 1)) 
		{
		// Calculate and update movement in polar coordinates /

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		// Calculate corresponding movement in cartesian coordinates /
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		// Update cartesian positions with respect to time /
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		// Calculate polar position from cartesian position /
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		// If position is within translocateable distance from nucleus /
			if(xmemory->posr > (nuclradius - 50))
			{
				// Translocate out of nucleus into cytoplasm /
				xmemory->posr = (xmemory->posr + 60);
				// update compartmentid 0 = cyto, 1 = nucleus/
				set_compartmentid(0);
				// Update new position /
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement /
				movex = -movex;
				movey = -movey;
				movez = -movez;*/
				// Update new movement /
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		
		/* If ALP mRNA bound to Ribosome*/
	if(( xmemory->state == 111 ) || (xmemory->state == 112))
	{
		//stay in position/
	}
 

	
	return 0;
}	

int BSP_outputdata()
{
		/* Send location message */
	add_BSPlocation_message(get_id(),		/* agent id */
						get_posx(),		/* agent x-axis position */
						get_posy(),		/* agent y-axis position */
						get_posz(),		/* agent z-axis position */
						get_state());	/* agent type and state */
						
	/* if BSP is unbound free in cytoplasm*/
	
	/* if BSPmRNA is bound to Ribosome and in unavailable state - BSP(122) */
	if(get_state() == 122)
	{
		if(get_ACS31() > 0)
		{
			/* Then decrement ACS31 counter */
			set_ACS31(get_ACS31() - 1);
			/* If processing has finished (ACS31 is zero) */
			if(get_ACS31() == 0)
			{
			/* Switch to bound available BSP mRNA and set ACS32 to 120 (typical time for translation - based on the literature*/
				set_state(121);
				set_ACS32(75);		
			}
		}
		/* ACS31 will only reset when a bond between Ribosome and BSP has been formed*/
	}

		/* if BSPmRNA is bound to Ribosome and in available state - BSP(121) */
	if(get_state() == 121)
	{
		if(get_ACS32() > 0)
		{
			/* Then decrement ACS32 counter */
			set_ACS32(get_ACS32() - 1);
			/* If processing has finished (ACS32 is zero) */
			if(get_ACS32() == 0)
			{
			/* Switch to unbound BSP mRNA */
				set_state(120);		
			}
		}
		/* ACS32 will only reset when BSP switches from unavailable to available bound*/
	}
	
	/* if mRNA exists and disintegration time is > 0 */
	if(get_disintegrationtime() > 0)
	{
		
			/* Then decrement disintegrationtime counter */
			set_disintegrationtime(get_disintegrationtime() - 1);
			/* If processing has finished (disintegrationtime is zero) */
			if(get_disintegrationtime() == 0)
			{
			/* remove mRNA agent from model, i.e death of particular mRNA agent */
				return 1;		
			}
		/* Disintegration time resets only when mRNA agent is produced*/
	}
	
	return 0;
}	

int BSP_checkbondtries()
{
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	
	xmachine_memory_BSP * xmemory = current_xmachine->xmachine_BSP;
	
	double closestdist = 0.0;
	int closestid = xmemory->id; // sham value
	int closeststate = 0;
	
	/* Check if I have been bound to Ribosome */
	/* Get first bond message from Ribosome agent*/
	Ribosomenewbond_message = get_first_Ribosomenewbond_message();
	/* And loop through messages until there are none left */
	while(Ribosomenewbond_message)
	{
		if(Ribosomenewbond_message->idto == get_id())
		{
			if((closestid == xmemory->id) || (Ribosomenewbond_message->distance < closestdist))
			{
				closestdist = Ribosomenewbond_message->distance;
				closestid = Ribosomenewbond_message->idfrom;
				closeststate = Ribosomenewbond_message->statefrom;
			}
		}
		
		/* Get next trybond message */
		Ribosomenewbond_message = get_next_Ribosomenewbond_message(Ribosomenewbond_message);
	}
	
	
	// if not sham value then bond created
	if(closestid != xmemory->id)
	{
		
		
						// If BSP mRNA is available to bind and if Ribosome is available to bind  
						if(((get_state() == 120)||(get_state() == 121))  && ((closeststate == 130)||(closeststate == 140))) 
						{
							set_state(122);											
							set_ACS31(20);
						}
						/* Send bond message to molecule to bind */
		add_BSPfinalbond_message(xmemory->id,
						xmemory->state,
						closestid,          // id of Ribosome agent that is involved in this particular bind/
						closeststate,       //state of Ribosome agent that is involved in this particular bind/
						get_posx(),
						get_posy(),
						get_posz());

	}

	return 0;
}

int BSP_move()
{
	xmachine_memory_BSP * xmemory = current_xmachine->xmachine_BSP;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving BSP mRNA in cytoplam*/
	if(( xmemory->state == 120) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Mirror position back into cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
	
	// If freely moving BSP mRNA in nucleus
	if(( xmemory->state == 120 ) && (xmemory->compartmentid == 1)) 
		{
		// Calculate and update movement in polar coordinates /

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		// Calculate corresponding movement in cartesian coordinates /
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		// Update cartesian positions with respect to time /
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		// Calculate polar position from cartesian position /
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		// If position is within translocateable distance from nucleus /
			if(xmemory->posr > (nuclradius - 50))
			{
				// Translocate out of nucleus into cytoplasm /
				xmemory->posr = (xmemory->posr + 60);
				// update compartmentid 0 = cyto, 1 = nucleus/
				set_compartmentid(0);
				// Update new position /
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement /
				movex = -movex;
				movey = -movey;
				movez = -movez;*/
				// Update new movement /
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		
		/* If BSP mRNA bound to Ribosome*/
	if(( xmemory->state == 121 ) || (xmemory->state == 122))
	{
		//stay in position/
	}		
	
	return 0;
}	

int OCN_generate()
{
	double cellSize = 1000.0;    /* approximate cell radius in nano meters */
	double nucleusSize = 400.0;     /* approximate cell nucleus radius in nano meters */

	int cyto_pErk_count = 0;
	//loop through the messages to count how many Erks are active in the cytoplasm
	// Get first availability message from Erk agent that is active and in cytoplasm/
	Erkavailability_message = get_first_Erkavailability_message();
	// And loop through messages until there are none left /
	while(Erkavailability_message)
	{
		if(Erkavailability_message->state == 52)
		{
			cyto_pErk_count += 1;
		}
		
		/* Get next Erk availability message */
		Erkavailability_message = get_next_Erkavailability_message(Erkavailability_message);
	}
	
	if(get_OCNACS() > 0)
	{
		set_OCNACS(get_OCNACS() - 1);
	}
	if((cyto_pErk_count > 400) && (get_OCNACS() == 0))
	{
		set_OCNgeneratedelay(get_OCNgeneratedelay() + 700);
		set_OCNACS(100);
	}
	if((get_OCNgeneratedelay() > 0) && (cyto_pErk_count > 300))
	{
		// Then decrement OCNgeneratedelay counter /
		set_OCNgeneratedelay(get_OCNgeneratedelay() - 1);
		// If processing has finished (OCNgeneratedelay is zero) /
			if((get_OCNgeneratedelay() % 700) == 0)
			{
				//randomly generate parameters for OCNmRNA to be produced in nucleus/
				srand ( time(NULL) );
				double speed_range = 10.0;
				double speed_ave = 20.0;
				double movetheta = srand()/(double)(RAND_MAX) * 2 * pi;
				double movephi = srand()/(double)(RAND_MAX) * 2 * pi;
				double mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
				double posr = ((0) +(srand()/(double)(RAND_MAX) * (nucleusSize - (75))));
				double postheta = srand()/(double)(RAND_MAX) * 2 * pi;
				double posphi = srand()/(double)(RAND_MAX) * 2 * pi;
				double posx = posr * cos(posphi) * cos(postheta);
				double posy = posr * cos(posphi) * sin(postheta);
				double posz = posr * sin(posphi);
				int OCNACStime = ((srand() % 100) + 100 );
				int disintegrationtime = (2500 +(srand() % 1000));
			// generate new OCNmRNA via RSK - Atf4 phosphorylation pathway/
				 add_OCN_agent(get_OCNproductid(),
							   postheta,
							   posphi,
							   posr,
							   posx,
							   posy,
							   posz,
							   movetheta,
							   movephi,
							   mover,
							   0,
							   0,
							   100,
							   disintegrationtime,
							   1);
				set_OCNproductid(get_OCNproductid()+1);
				set_OCNACS(OCNACStime);
			}
	}
	
	return 0;
}

int Ribosome_outputdata() 
{
						
	/* if incomplete ribosome is bound to mRNA - Ribosome (131) */
	if(get_state() == 131)
	{
		if(get_ACS33() > 0)
		{
			/* Then decrement ACS33 counter */
			set_ACS33(get_ACS33() - 1);
			/* If processing has finished (ACS33 is zero) */
			if(get_ACS33() == 0)
			{
			/* Switch to incomplete unbound Ribosome - Ribosome (130)*/
				set_state(130);		
			}
		}
		/* ACS33 will only reset when a bond between incomplete ribosome and mRNA is formed*/
	}	
	
	/* if complete ribosome is bound to mRNA - Ribosome (141) */
	if(get_state() == 141)
	{
		if(get_ACS34() > 0)
		{
			/* Then decrement ACS34 counter */
			set_ACS34(get_ACS34() - 1);
			/* If processing has finished (ACS34 is zero) */
			if(get_ACS34() == 0)
			{
			/* Switch to complete unbound Ribosome - Ribosome (140)*/
				set_state(140);		
			}
		}
		/* ACS33 will only reset when a bond between incomplete ribosome and mRNA is formed*/
	}	
	
		
	return 0;
}	

int Ribosome_inputdata()
{
	xmachine_memory_Ribosome * xmemory = current_xmachine->xmachine_Ribosome;
	/* Create variables to use */
	double x1, y1, x2, y2, z1, z2, closestdist;
	int id2, s1, s2, closestid, closeststate; 
	double distance;
		
	/* Copy agent data to #1 variables */
	x1 = get_posx();
	y1 = get_posy();
	z1 = get_posz();
	s1 = get_state();
	
	/* Interaction radius of the agent (squared so don't need to sqrt compared variable) */
	closestdist = get_iradius()*get_iradius();
	/* Index of closest agent to interact with and set to default as own id */
	closestid = get_id();

	// Check for possible bindings /
	// Get first location message /
	OCNlocation_message = get_first_OCNlocation_message();
	// And loop through them until there are none left /
	
	while(OCNlocation_message) 
	{
		// If the location message is not from the current agent /
		if((OCNlocation_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = OCNlocation_message->id;
			x2 = OCNlocation_message->x;
			y2 = OCNlocation_message->y;
			z2 = OCNlocation_message->z;
			s2 = OCNlocation_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				// if agents Ribosome and mRNA are in states able to bind /
			if(((s1 == 130) || (s1 == 140))  && ((s2 % 10) != 2)) 				
				{
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		// Get next location message /
		OCNlocation_message = get_next_OCNlocation_message(OCNlocation_message);
	}	
	
	// Check for possible bindings /
	// Get first location message /
	OPNlocation_message = get_first_OPNlocation_message();
	// And loop through them until there are none left /
	
	while(OPNlocation_message) 
	{
		// If the location message is not from the current agent /
		if((OPNlocation_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = OPNlocation_message->id;
			x2 = OPNlocation_message->x;
			y2 = OPNlocation_message->y;
			z2 = OPNlocation_message->z;
			s2 = OPNlocation_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				// if agents Ribosome and mRNA are in states able to bind /
			if(((s1 == 130) || (s1 == 140))  && ((s2 % 10) != 2))				
				{
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		// Get next location message /
		OPNlocation_message = get_next_OPNlocation_message(OPNlocation_message);
	}	
	
	// Check for possible bindings /
	// Get first location message /
	ALPlocation_message = get_first_ALPlocation_message();
	// And loop through them until there are none left /
	
	while(ALPlocation_message) 
	{
		// If the location message is not from the current agent /
		if((ALPlocation_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = ALPlocation_message->id;
			x2 = ALPlocation_message->x;
			y2 = ALPlocation_message->y;
			z2 = ALPlocation_message->z;
			s2 = ALPlocation_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				// if agents Ribosome and mRNA are in states able to bind /
			if(((s1 == 130) || (s1 == 140))  && ((s2 % 10) != 2)) 				
				{
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		// Get next location message /
		ALPlocation_message = get_next_ALPlocation_message(ALPlocation_message);
	}	
	
	// Check for possible bindings /
	// Get first location message /
	BSPlocation_message = get_first_BSPlocation_message();
	// And loop through them until there are none left /
	
	while(BSPlocation_message) 
	{
		// If the location message is not from the current agent /
		if((BSPlocation_message->id != get_id()))
		{
			// Copy location message data to #2 variables /
			id2 = BSPlocation_message->id;
			x2 = BSPlocation_message->x;
			y2 = BSPlocation_message->y;
			z2 = BSPlocation_message->z;
			s2 = BSPlocation_message->state;
			
			// Calculate the distance between agent and message sending agent /
			// Do not square root as more efficient to square interaction radius /
			distance = (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2);
			// If distance is equal or smaller than the agent interaction radius /
			if(distance < closestdist)
			{
				// if agents Ribosome and mRNA are in states able to bind /
			if(((s1 == 130) || (s1 == 140))  && ((s2 % 10) != 2)) 				
				{
					// if this agent is closer than the index of the closest /
					if(distance < closestdist)
					{
						// update closest distance to bondable agent /
						closestdist = distance;
						// also update the closest agent's id and state /
						closestid = id2;
						closeststate = s2;
					}
				}
			}
		}
		// Get next location message /
		BSPlocation_message = get_next_BSPlocation_message(BSPlocation_message);
	}	
	
	// If closest id is not the sham value therefore a possible bond has been found /
	if(closestid != xmemory->id)
	{
		// Send try to bond message /
		add_Ribosomenewbond_message(get_id(),         
						get_state(),
						closestid,           // id of agent message is refering to /
						closeststate,       // state of agent message is referring to/
						closestdist,
						get_iradius(),
						get_posx(),
						get_posy(),
						get_posz());
	}
	
	return 0;
}


int Ribosome_movetranslate()
{
	xmachine_memory_Ribosome * xmemory = current_xmachine->xmachine_Ribosome;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving complete or incomplete Ribosome in cytoplam*/
	if(( xmemory->state == 130 || xmemory->state == 140 ))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > (cellradius - 200))
			{
				/* Mirror position back into compartment */
				xmemory->posr = (2*(cellradius - 200))-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
		/* If complete or incomplete Ribosome bound to mRNA*/
		if((xmemory->state == 131 || xmemory->state == 141))
		{
			//stay in position bound to agent/
		}
		
	/* Check if I have been bound to any mRNA */
	
	/* Get first bond message */
	OCNfinalbond_message = get_first_OCNfinalbond_message();
	/* And loop through messages until there are none left */
	while(OCNfinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(OCNfinalbond_message->idto == xmemory->id )
		{

		/* If agent Ribosome is incomplete and available to bind*/
		if( get_state() == 130)
						{
						set_state(131);
						set_ACS33(100); //arbitrary
						//reposition ribosome to position of mRNA
						set_posx(OCNfinalbond_message->x);
						set_posy(OCNfinalbond_message->y);
						set_posz(OCNfinalbond_message->z);
						}
						
		/* If agent Ribosome is complete and available to bind*/
		if( get_state() == 140)
						{
						set_state(141);
						set_ACS34(100); //arbitrary
						//reposition ribosome to position of mRNA
						set_posx(OCNfinalbond_message->x);
						set_posy(OCNfinalbond_message->y);
						set_posz(OCNfinalbond_message->z);
						
						//add matrix protein agents here/
						set_OCNproteindelay(95);
						}
		}
		/* Get next bond message */
		OCNfinalbond_message = get_next_OCNfinalbond_message(OCNfinalbond_message);			
	}
	
	/* Get first bond message */
	OPNfinalbond_message = get_first_OPNfinalbond_message();
	/* And loop through messages until there are none left */
	while(OPNfinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(OPNfinalbond_message->idto == xmemory->id )
		{

		/* If agent Ribosome is incomplete and available to bind*/
		if( get_state() == 130)
						{
						set_state(131);
						set_ACS33(100); //arbitrary
						//reposition ribosome to position of mRNA
						set_posx(OPNfinalbond_message->x);
						set_posy(OPNfinalbond_message->y);
						set_posz(OPNfinalbond_message->z);
						}
						
		/* If agent Ribosome is complete and available to bind*/
		if( get_state() == 140)
						{
						set_state(141);
						set_ACS34(100); //arbitrary
						//reposition ribosome to position of mRNA
						set_posx(OPNfinalbond_message->x);
						set_posy(OPNfinalbond_message->y);
						set_posz(OPNfinalbond_message->z);
						
						//add matrix protein agents here /
						set_OPNproteindelay(95);
						}
		}
		/* Get next bond message */
		OPNfinalbond_message = get_next_OPNfinalbond_message(OPNfinalbond_message);			
	}
	
	/* Get first bond message */
	ALPfinalbond_message = get_first_ALPfinalbond_message();
	/* And loop through messages until there are none left */
	while(ALPfinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(ALPfinalbond_message->idto == xmemory->id )
		{

		/* If agent Ribosome is incomplete and available to bind*/
		if( get_state() == 130)
						{
						set_state(131);
						set_ACS33(100); //arbitrary
						//reposition ribosome to position of mRNA
						set_posx(ALPfinalbond_message->x);
						set_posy(ALPfinalbond_message->y);
						set_posz(ALPfinalbond_message->z);
						}
						
		/* If agent Ribosome is complete and available to bind*/
		if( get_state() == 140)
						{
						set_state(141);
						set_ACS34(100); //arbitrary
						//reposition ribosome to position of mRNA
						set_posx(ALPfinalbond_message->x);
						set_posy(ALPfinalbond_message->y);
						set_posz(ALPfinalbond_message->z);
						
						//add matrix protein agents here /
						set_ALPproteindelay(95);
						}
		}
		/* Get next bond message */
		ALPfinalbond_message = get_next_ALPfinalbond_message(ALPfinalbond_message);			
	}
	
	/* Get first bond message */
	BSPfinalbond_message = get_first_BSPfinalbond_message();
	/* And loop through messages until there are none left */
	while(BSPfinalbond_message)
	{
		/* bond message refers to me (same id) and tag refers to a bind */
		if(BSPfinalbond_message->idto == xmemory->id )
		{

		/* If agent Ribosome is incomplete and available to bind*/
		if( get_state() == 130)
						{
						set_state(131);
						set_ACS33(100); //arbitrary
						//reposition ribosome to position of mRNA
						set_posx(BSPfinalbond_message->x);
						set_posy(BSPfinalbond_message->y);
						set_posz(BSPfinalbond_message->z);
						}
						
		/* If agent Ribosome is complete and available to bind*/
		if( get_state() == 140)
						{
						set_state(141);
						set_ACS34(100); //arbitrary
						//reposition ribosome to position of mRNA
						set_posx(BSPfinalbond_message->x);
						set_posy(BSPfinalbond_message->y);
						set_posz(BSPfinalbond_message->z);
						
						//add matrix protein agents here - by setting up BSPprotein generation delay counter/
						set_BSPproteindelay(95);
						}
		}
		/* Get next bond message */
		BSPfinalbond_message = get_next_BSPfinalbond_message(BSPfinalbond_message);			
	}
		
	if(get_state() == 141)
	{
		if(get_OCNproteindelay() > 0)
		{
			/* Then decrement OCNproteindelay counter */
			set_OCNproteindelay(get_OCNproteindelay() - 1);
			/* If processing has finished (OCNproteindelay is zero) */
			if(get_OCNproteindelay() == 0)
			{
			//Generate OCN Matrix Protein
				//randomly generate parameters for OCN matrix protein to be produced in ctoplasm/
				srand ( time(NULL) );
				double speed_range = 10.0;
				double speed_ave = 20.0;
				double movetheta = srand()/(double)(RAND_MAX) * 2 * pi;
				double movephi = srand()/(double)(RAND_MAX) * 2 * pi;
				double mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
			// generate new OCN matrix protein agents/
				 add_OCNprotein_agent(get_OCNproteinid(), 
							   get_postheta(),
							   get_posphi(),
							   get_posr(),
							   get_posx(),
							   get_posy(),
							   get_posz(),
							   movetheta,
							   movephi,
							   mover,
							   150, //state - being transported out state/
							   0); // compartment id - 0 for cytosol, 2 for ECM/
				set_OCNproteinid(get_OCNproteinid()+1);	
			}
		}
		
		if(get_OPNproteindelay() > 0)
		{
			/* Then decrement OPNproteindelay counter */
			set_OPNproteindelay(get_OPNproteindelay() - 1);
			/* If processing has finished (OPNproteindelay is zero) */
			if(get_OPNproteindelay() == 0)
			{
			//Generate OPN Matrix Protein
				//randomly generate parameters for OPN matrix protein to be produced in cytoplasm/
				srand ( time(NULL) );
				double speed_range = 10.0;
				double speed_ave = 20.0;
				double movetheta = srand()/(double)(RAND_MAX) * 2 * pi;
				double movephi = srand()/(double)(RAND_MAX) * 2 * pi;
				double mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
			// generate new OPN matrix protein agents/
				 add_OPNprotein_agent(get_OPNproteinid(),
							   get_postheta(),
							   get_posphi(),
							   get_posr(),
							   get_posx(),
							   get_posy(),
							   get_posz(),
							   movetheta,
							   movephi,
							   mover,
							   150, //state - being transported out state/
							   0); // compartment id - 0 for cytosol, 2 for ECM/
				set_OPNproteinid(get_OPNproteinid()+1);	
			}
		}
		
		if(get_ALPproteindelay() > 0)
		{
			/* Then decrement ALPproteindelay counter */
			set_ALPproteindelay(get_ALPproteindelay() - 1);
			/* If processing has finished (ALPproteindelay is zero) */
			if(get_ALPproteindelay() == 0)
			{
			//Generate ALP Matrix Protein
				//randomly generate parameters for ALP matrix protein to be produced in cytoplasm/
				srand ( time(NULL) );
				double speed_range = 10.0;
				double speed_ave = 20.0;
				double movetheta = srand()/(double)(RAND_MAX) * 2 * pi;
				double movephi = srand()/(double)(RAND_MAX) * 2 * pi;
				double mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
			// generate new ALP matrix protein agents/
				 add_ALPprotein_agent(get_ALPproteinid(),
							   get_postheta(),
							   get_posphi(),
							   get_posr(),
							   get_posx(),
							   get_posy(),
							   get_posz(),
							   movetheta,
							   movephi,
							   mover,
							   150, //state - being transported out state/
							   0); // compartment id - 0 for cytosol, 2 for ECM/
				set_ALPproteinid(get_ALPproteinid()+1);	
			}
		}
		
		if(get_BSPproteindelay() > 0)
		{
			/* Then decrement BSPproteindelay counter */
			set_BSPproteindelay(get_BSPproteindelay() - 1);
			/* If processing has finished (BSPproteindelay is zero) */
			if(get_BSPproteindelay() == 0)
			{
			//Generate BSP Matrix Protein
				//randomly generate parameters for BSP matrix protein to be produced in cytoplasm/
				srand ( time(NULL) );
				double speed_range = 10.0;
				double speed_ave = 20.0;
				double movetheta = srand()/(double)(RAND_MAX) * 2 * pi;
				double movephi = srand()/(double)(RAND_MAX) * 2 * pi;
				double mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
			// generate new BSP matrix protein agents/
				 add_BSPprotein_agent(get_BSPproteinid(),
							   get_postheta(),
							   get_posphi(),
							   get_posr(),
							   get_posx(),
							   get_posy(),
							   get_posz(),
							   movetheta,
							   movephi,
							   mover,
							   150, //state - being transported out state/
							   0); // compartment id - 0 for cytosol, 2 for ECM/
				set_BSPproteinid(get_BSPproteinid()+1);	
			}
		}
	}
	
	
	return 0;
}	

int OCNprotein_move()
{
		xmachine_memory_OCNprotein * xmemory = current_xmachine->xmachine_OCNprotein;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving OCNprotein in cytoplam*/
	if(( xmemory->state == 150) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Reposition out of cell */
				xmemory->posr = (xmemory->posr + 50);
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
				//update compartment id/
				set_compartmentid(2);
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
	
	// If freely moving OCNprotein outside cell
	if(( xmemory->state == 150 ) && (xmemory->compartmentid == 2)) 
		{
		// Calculate and update movement in polar coordinates /

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		// Calculate corresponding movement in cartesian coordinates /
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		// Update cartesian positions with respect to time /
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		// Calculate polar position from cartesian position /
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		// If position is within depositable distance from cellmembrane /
			if(xmemory->posr > (cellradius + 100))
			{
				// maintain position /
				xmemory->posr = (xmemory->posr);
				// update state to deposited outside cell state/
				set_state(151);
				// Update new movement /
				xmemory->movetheta = 0;
				xmemory->movephi = 0;
				xmemory->mover = 0;
			}
		//if position is beyond cell membrane /
		if(xmemory->posr < cellradius)
			{
				/* Mirror position back outside of cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		
		/* If OCNprotein is deposited*/
	if(( xmemory->state == 151 ))
	{
		//stay in position/
	}	
	
	/* Send depositedOCNcount message */
	add_depositedOCNcount_message(get_id(),		/* agent id */
						get_state());		/* agent type and state */
						
	return 0;
}

int OPNprotein_move()
{
	xmachine_memory_OPNprotein * xmemory = current_xmachine->xmachine_OPNprotein;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving OPNprotein in cytoplam*/
	if(( xmemory->state == 150) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Reposition out of cell */
				xmemory->posr = (xmemory->posr + 50);
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
				//update compartment id/
				set_compartmentid(2);
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
	
	// If freely moving OPNprotein outside cell
	if(( xmemory->state == 150 ) && (xmemory->compartmentid == 2)) 
		{
		// Calculate and update movement in polar coordinates /

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		// Calculate corresponding movement in cartesian coordinates /
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		// Update cartesian positions with respect to time /
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		// Calculate polar position from cartesian position /
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		// If position is within depositable distance from cellmembrane /
			if(xmemory->posr > (cellradius + 100))
			{
				// maintain position /
				xmemory->posr = (xmemory->posr);
				// update state to deposited outside cell state/
				set_state(151);
				// Update new movement /
				xmemory->movetheta = 0;
				xmemory->movephi = 0;
				xmemory->mover = 0;
			}
		//if position is beyond cell membrane /
		if(xmemory->posr < cellradius)
			{
				/* Mirror position back outside of cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		
		/* If OPNprotein is deposited*/
	if(( xmemory->state == 151 ))
	{
		//stay in position/
	}	
	
	/* Send depositedOPNcount message */
	add_depositedOPNcount_message(get_id(),		/* agent id */
						get_state());		/* agent type and state */
	
	return 0;
}

int ALPprotein_move()
{
		xmachine_memory_ALPprotein * xmemory = current_xmachine->xmachine_ALPprotein;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;        /* Average speed of protein agent */
	double speed_range = 1;      /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving ALPprotein in cytoplam*/
	if(( xmemory->state == 150) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Reposition out of cell */
				xmemory->posr = (xmemory->posr + 50);
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
				//update compartment id/
				set_compartmentid(2);
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
	
	// If freely moving ALPprotein outside cell
	if(( xmemory->state == 150 ) && (xmemory->compartmentid == 2)) 
		{
		// Calculate and update movement in polar coordinates /

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		// Calculate corresponding movement in cartesian coordinates /
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		// Update cartesian positions with respect to time /
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		// Calculate polar position from cartesian position /
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		// If position is within depositable distance from cellmembrane /
			if(xmemory->posr > (cellradius + 100))
			{
				// maintain position /
				xmemory->posr = (xmemory->posr);
				// update state to deposited outside cell state/
				set_state(151);
				// Update new movement /
				xmemory->movetheta = 0;
				xmemory->movephi = 0;
				xmemory->mover = 0;
			}
		//if position is beyond cell membrane /
		if(xmemory->posr < cellradius)
			{
				/* Mirror position back outside of cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		
		/* If ALPprotein is deposited*/
	if(( xmemory->state == 151 ))
	{
		//stay in position/
	}	
	
	/* Send depositedALPcount message */
	add_depositedALPcount_message(get_id(),		/* agent id */
						get_state());		/* agent type and state */
	
	return 0;
}

int BSPprotein_move()
{
		xmachine_memory_BSPprotein * xmemory = current_xmachine->xmachine_BSPprotein;
	double cellradius = 1000.0;    /* approximate cell radius in nano meters */
	double nuclradius = 400.0;     /* approximate cell nucleus radius in nano meters */
	double dt = 10.0;                /* Time differential */
	/* Molecule movement constants */
	double speed_ave = 2;         /* Average speed of protein agent */
	double speed_range = 1;       /* Range of speed, + or - from average speed */
	double angle_range = pi/10.0;   /* Angle range in radians, + or - from current angle */
	/* Variables for calculations */
	double movex;
	double movey;
	double movez;


	/* If freely moving BSPprotein in cytoplam*/
	if(( xmemory->state == 150) && (xmemory->compartmentid == 0))    															
					
	{
		/* Calculate and update movement in polar coordinates */

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		/* Calculate corresponding movement in cartesian coordinates */
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		/* Update cartesian positions with respect to time */
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		/* Calculate polar position from cartesian position */
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		/* If position is beyond the cell membrane */
			if(xmemory->posr > cellradius)
			{
				/* Reposition out of cell */
				xmemory->posr = (xmemory->posr + 50);
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
				//update compartment id/
				set_compartmentid(2);
			}
			/* If position is beyond the nuclear membrane */
			if(xmemory->posr < nuclradius)
			{
				/* Mirror position back into cytoplasm */
				xmemory->posr = (2*nuclradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
			
	}
	
	// If freely moving BSPprotein outside cell
	if(( xmemory->state == 150 ) && (xmemory->compartmentid == 2)) 
		{
		// Calculate and update movement in polar coordinates /

		xmemory->movetheta = xmemory->movetheta + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->movephi = xmemory->movephi + (-angle_range + 2.0*angle_range*((double)srand()/RAND_MAX));
		xmemory->mover = speed_ave + (-speed_range + 2.0*speed_range*((double)srand()/RAND_MAX));
		// Calculate corresponding movement in cartesian coordinates /
		movex = xmemory->mover * cos(xmemory->movephi) * cos(xmemory->movetheta);
		movey = xmemory->mover * cos(xmemory->movephi) * sin(xmemory->movetheta);
		movez = xmemory->mover * sin(xmemory->movephi);
		// Update cartesian positions with respect to time /
		xmemory->posx += movex * dt;
		xmemory->posy += movey * dt;
		xmemory->posz += movez * dt;
		// Calculate polar position from cartesian position /
		xmemory->postheta = atan2(xmemory->posy, xmemory->posx);
		xmemory->posphi = atan2(xmemory->posz,sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)));
		xmemory->posr = sqrt((xmemory->posx*xmemory->posx)+(xmemory->posy*xmemory->posy)+(xmemory->posz*xmemory->posz));
		
		// If position is within depositable distance from cellmembrane /
			if(xmemory->posr > (cellradius + 100))
			{
				// maintain position /
				xmemory->posr = (xmemory->posr);
				// update state to deposited outside cell state/
				set_state(151);
				// Update new movement /
				xmemory->movetheta = 0;
				xmemory->movephi = 0;
				xmemory->mover = 0;
			}
		//if position is beyond cell membrane /
		if(xmemory->posr < cellradius)
			{
				/* Mirror position back outside of cell */
				xmemory->posr = (2*cellradius)-xmemory->posr;
				/* Update new position */
				xmemory->posx = xmemory->posr * cos(xmemory->posphi) * cos(xmemory->postheta);
				xmemory->posy = xmemory->posr * cos(xmemory->posphi) * sin(xmemory->postheta);
				xmemory->posz = xmemory->posr * sin(xmemory->posphi);
				/* Mirror direction of movement */
				movex = -movex;
				movey = -movey;
				movez = -movez;
				/* Update new movement */
				xmemory->movetheta = atan2(movey, movex);
				xmemory->movephi = atan2(movez,sqrt((movex*movex)+(movey*movey)));
				xmemory->mover = sqrt((movex*movex)+(movey*movey)+(movez*movez));
			}
	}		
		
		/* If BSPprotein is deposited*/
	if(( xmemory->state == 151 ))
	{
		//stay in position/
	}	
	
	/* Send depositedBSPcount message */
	add_depositedBSPcount_message(get_id(),		/* agent id */
						get_state());		/* agent type and state */
	
	return 0;
}

int update_elasticmoduli()
{
	int baseEcell = 100;//the initial elastic modulus of the cell (at t0)
	int baseEmatrix = 10;//the initial elastic modulus of the extracellular matrix (at t0)
	int deposited_OCN = 0;
	int deposited_OPN = 0;
	int deposited_ALP = 0;
	int deposited_BSP = 0;
	
	//loop through the messages to count how many of each of the matrix proteins have been deposited
	
	// Get first message from deposited OCN agents/
	depositedOCNcount_message = get_first_depositedOCNcount_message();
	// And loop through messages until there are none left /
	while(depositedOCNcount_message)
	{
		if(depositedOCNcount_message->state == 151)
		{
			deposited_OCN += 1;
		}
		
		/* Get next deposited OCN count message */
		depositedOCNcount_message = get_next_depositedOCNcount_message(depositedOCNcount_message);
	}
	
	// Get first message from deposited OPN agents/
	depositedOPNcount_message = get_first_depositedOPNcount_message();
	// And loop through messages until there are none left /
	while(depositedOPNcount_message)
	{
		if(depositedOPNcount_message->state == 151)
		{
			deposited_OPN += 1;
		}
		
		/* Get next deposited OPN count message */
		depositedOPNcount_message = get_next_depositedOPNcount_message(depositedOPNcount_message);
	}
	
	// Get first message from deposited ALP agents/
	depositedALPcount_message = get_first_depositedALPcount_message();
	// And loop through messages until there are none left /
	while(depositedALPcount_message)
	{
		if(depositedALPcount_message->state == 151)
		{
			deposited_ALP += 1;
		}
		
		/* Get next deposited ALP count message */
		depositedALPcount_message = get_next_depositedALPcount_message(depositedALPcount_message);
	}
	
	// Get first message from deposited BSP agents/
	depositedBSPcount_message = get_first_depositedBSPcount_message();
	// And loop through messages until there are none left /
	while(depositedBSPcount_message)
	{
		if(depositedBSPcount_message->state == 151)
		{
			deposited_BSP += 1;
		}
		
		/* Get next deposited BSP count message */
		depositedBSPcount_message = get_next_depositedBSPcount_message(depositedBSPcount_message);
	}
	
	//here we can define the roles each matrix protein will play
	//i.e how much change will there be in the Elastic modulus of the ECM when a specific protein is deposited upto a certain number etc
	// the rules at the moment are just arbitrary and need to be adjusted based on literature correlating elastic modulus or stiffness increase with matrix production
	if(deposited_OCN  > 0)
	{
		set_ECMelasticmod((baseEmatrix + (deposited_OCN * 5))); //linear increase
	}
	
	if(deposited_OCN  > 10)
	{
		set_ECMelasticmod((get_ECMelasticmod() * 1.002)); //nonlinear increase after 10 OCN proteins deposited
	}
	
	if(deposited_OPN  > 0)
	{
		set_ECMelasticmod((baseEmatrix + (deposited_OPN * 3))); //linear increase
	}
	
	if(deposited_OPN  > 10)
	{
		set_ECMelasticmod((get_ECMelasticmod() * 1.001)); //nonlinear increase after 10 OCN proteins deposited
	}
	
	return 0;
}